# coding: utf-8
"""Test that tokenizer exceptions are parsed correctly."""


from __future__ import unicode_literals

import pytest


@pytest.mark.parametrize('text,norms', [("пн.", ["понедельник"]),
                                        ("пт.", ["пятница"]),
                                        ("дек.", ["декабрь"])])
def test_ru_tokenizer_abbrev_exceptions(ru_tokenizer, text, norms):
    tokens = ru_tokenizer(text)
    assert len(tokens) == 1
    assert [token.norm_ for token in tokens] == norms



# coding: utf-8
from __future__ import unicode_literals
import pytest


word2vec_str = """, -0.046107 -0.035951 -0.560418
de -0.648927 -0.400976 -0.527124
. 0.113685 0.439990 -0.634510
\u00A0 -1.499184 -0.184280 -0.598371"""


@pytest.mark.xfail
def test_issue834(en_vocab, text_file):
    """Test that no-break space (U+00A0) is detected as space by the load_vectors function."""
    text_file.write(word2vec_str)
    text_file.seek(0)
    vector_length = en_vocab.load_vectors(text_file)
    assert vector_length == 3



# coding: utf8
from __future__ import unicode_literals

from ..symbols import ORTH, ENT_TYPE, LOWER


ENT_ID = "ent_id"
ENTITY_RULES = []


for name, tag, patterns in [
    ("Reddit", "PRODUCT", [[{LOWER: "reddit"}]]),
    ("Linux", "PRODUCT", [[{LOWER: "linux"}]]),
    ("Haskell", "PRODUCT", [[{LOWER: "haskell"}]]),
    ("HaskellCurry", "PERSON", [[{LOWER: "haskell"}, {LOWER: "curry"}]]),
    ("Javascript", "PRODUCT", [[{LOWER: "javascript"}]]),
    ("CSS", "PRODUCT", [[{LOWER: "css"}], [{LOWER: "css3"}]]),
    ("HTML", "PRODUCT", [[{LOWER: "html"}], [{LOWER: "html5"}]]),
    ("Python", "PRODUCT", [[{ORTH: "Python"}]]),
    ("Ruby", "PRODUCT", [[{ORTH: "Ruby"}]]),
    ("spaCy", "PRODUCT", [[{LOWER: "spacy"}]]),
    ("displaCy", "PRODUCT", [[{LOWER: "displacy"}]]),
    ("Digg", "PRODUCT", [[{LOWER: "digg"}]]),
    ("FoxNews", "ORG", [[{LOWER: "foxnews"}], [{LOWER: "fox"}, {LOWER: "news"}]]),
    ("Google", "ORG", [[{LOWER: "google"}]]),
    ("Mac", "PRODUCT", [[{LOWER: "mac"}]]),
    ("Wikipedia", "PRODUCT", [[{LOWER: "wikipedia"}]]),
    ("Windows", "PRODUCT", [[{LOWER: "windows"}]]),
    ("Dell", "ORG", [[{LOWER: "dell"}]]),
    ("Facebook", "ORG", [[{LOWER: "facebook"}]]),
    ("Blizzard", "ORG", [[{LOWER: "blizzard"}]]),
    ("Ubuntu", "ORG", [[{LOWER: "ubuntu"}]]),
    ("YouTube", "PRODUCT", [[{LOWER: "youtube"}]]),]:
    ENTITY_RULES.append({ENT_ID: name, 'attrs': {ENT_TYPE: tag}, 'patterns': patterns})


FALSE_POSITIVES = [
    [{ORTH: "Shit"}],
    [{ORTH: "Weed"}],
    [{ORTH: "Cool"}],
    [{ORTH: "Btw"}],
    [{ORTH: "Bah"}],
    [{ORTH: "Bullshit"}],
    [{ORTH: "Lol"}],
    [{ORTH: "Yo"}, {LOWER: "dawg"}],
    [{ORTH: "Yay"}],
    [{ORTH: "Ahh"}],
    [{ORTH: "Yea"}],
    [{ORTH: "Bah"}]
]



# cython: profile=True
# coding: utf8
from __future__ import unicode_literals, print_function

import re
import ujson
import random
import cytoolz
import itertools

from .syntax import nonproj
from .tokens import Doc
from .errors import Errors
from . import util
from .util import minibatch


def tags_to_entities(tags):
    entities = []
    start = None
    for i, tag in enumerate(tags):
        if tag is None:
            continue
        if tag.startswith('O'):
            # TODO: We shouldn't be getting these malformed inputs. Fix this.
            if start is not None:
                start = None
            continue
        elif tag == '-':
            continue
        elif tag.startswith('I'):
            if start is None:
                raise ValueError(Errors.E067.format(tags=tags[:i]))
            continue
        if tag.startswith('U'):
            entities.append((tag[2:], i, i))
        elif tag.startswith('B'):
            start = i
        elif tag.startswith('L'):
            entities.append((tag[2:], start, i))
            start = None
        else:
            raise ValueError(Errors.E068.format(tag=tag))
    return entities


def merge_sents(sents):
    m_deps = [[], [], [], [], [], []]
    m_brackets = []
    i = 0
    for (ids, words, tags, heads, labels, ner), brackets in sents:
        m_deps[0].extend(id_ + i for id_ in ids)
        m_deps[1].extend(words)
        m_deps[2].extend(tags)
        m_deps[3].extend(head + i for head in heads)
        m_deps[4].extend(labels)
        m_deps[5].extend(ner)
        m_brackets.extend((b['first'] + i, b['last'] + i, b['label'])
                          for b in brackets)
        i += len(ids)
    return [(m_deps, m_brackets)]


def align(cand_words, gold_words):
    cost, edit_path = _min_edit_path(cand_words, gold_words)
    alignment = []
    i_of_gold = 0
    for move in edit_path:
        if move == 'M':
            alignment.append(i_of_gold)
            i_of_gold += 1
        elif move == 'S':
            alignment.append(None)
            i_of_gold += 1
        elif move == 'D':
            alignment.append(None)
        elif move == 'I':
            i_of_gold += 1
        else:
            raise Exception(move)
    return alignment


punct_re = re.compile(r'\W')


def _min_edit_path(cand_words, gold_words):
    cdef:
        Pool mem
        int i, j, n_cand, n_gold
        int* curr_costs
        int* prev_costs

    # TODO: Fix this --- just do it properly, make the full edit matrix and
    # then walk back over it...
    # Preprocess inputs
    cand_words = [punct_re.sub('', w).lower() for w in cand_words]
    gold_words = [punct_re.sub('', w).lower() for w in gold_words]

    if cand_words == gold_words:
        return 0, ''.join(['M' for _ in gold_words])
    mem = Pool()
    n_cand = len(cand_words)
    n_gold = len(gold_words)
    # Levenshtein distance, except we need the history, and we may want
    # different costs. Mark operations with a string, and score the history
    # using _edit_cost.
    previous_row = []
    prev_costs = <int*>mem.alloc(n_gold + 1, sizeof(int))
    curr_costs = <int*>mem.alloc(n_gold + 1, sizeof(int))
    for i in range(n_gold + 1):
        cell = ''
        for j in range(i):
            cell += 'I'
        previous_row.append('I' * i)
        prev_costs[i] = i
    for i, cand in enumerate(cand_words):
        current_row = ['D' * (i + 1)]
        curr_costs[0] = i+1
        for j, gold in enumerate(gold_words):
            if gold.lower() == cand.lower():
                s_cost = prev_costs[j]
                i_cost = curr_costs[j] + 1
                d_cost = prev_costs[j + 1] + 1
            else:
                s_cost = prev_costs[j] + 1
                i_cost = curr_costs[j] + 1
                d_cost = prev_costs[j + 1] + (1 if cand else 0)

            if s_cost <= i_cost and s_cost <= d_cost:
                best_cost = s_cost
                best_hist = previous_row[j] + ('M' if gold == cand else 'S')
            elif i_cost <= s_cost and i_cost <= d_cost:
                best_cost = i_cost
                best_hist = current_row[j] + 'I'
            else:
                best_cost = d_cost
                best_hist = previous_row[j + 1] + 'D'

            current_row.append(best_hist)
            curr_costs[j+1] = best_cost
        previous_row = current_row
        for j in range(len(gold_words) + 1):
            prev_costs[j] = curr_costs[j]
            curr_costs[j] = 0

    return prev_costs[n_gold], previous_row[-1]


class GoldCorpus(object):
    """An annotated corpus, using the JSON file format. Manages
    annotations for tagging, dependency parsing and NER."""
    def __init__(self, train_path, dev_path, gold_preproc=True, limit=None):
        """Create a GoldCorpus.

        train_path (unicode or Path): File or directory of training data.
        dev_path (unicode or Path): File or directory of development data.
        RETURNS (GoldCorpus): The newly created object.
        """
        self.train_path = util.ensure_path(train_path)
        self.dev_path = util.ensure_path(dev_path)
        self.limit = limit
        self.train_locs = self.walk_corpus(self.train_path)
        self.dev_locs = self.walk_corpus(self.dev_path)

    @property
    def train_tuples(self):
        i = 0
        for loc in self.train_locs:
            gold_tuples = read_json_file(loc)
            for item in gold_tuples:
                yield item
                i += len(item[1])
                if self.limit and i >= self.limit:
                    break

    @property
    def dev_tuples(self):
        i = 0
        for loc in self.dev_locs:
            gold_tuples = read_json_file(loc)
            for item in gold_tuples:
                yield item
                i += len(item[1])
                if self.limit and i >= self.limit:
                    break

    def count_train(self):
        n = 0
        i = 0
        for raw_text, paragraph_tuples in self.train_tuples:
            n += sum([len(s[0][1]) for s in paragraph_tuples])
            if self.limit and i >= self.limit:
                break
            i += len(paragraph_tuples)
        return n

    def train_docs(self, nlp, gold_preproc=False,
                   projectivize=False, max_length=None,
                   noise_level=0.0):
        train_tuples = self.train_tuples
        if projectivize:
            train_tuples = nonproj.preprocess_training_data(
                self.train_tuples, label_freq_cutoff=100)
        random.shuffle(train_tuples)
        gold_docs = self.iter_gold_docs(nlp, train_tuples, gold_preproc,
                                        max_length=max_length,
                                        noise_level=noise_level)
        yield from gold_docs

    def dev_docs(self, nlp, gold_preproc=False):
        gold_docs = self.iter_gold_docs(nlp, self.dev_tuples, gold_preproc)
        yield from gold_docs

    @classmethod
    def iter_gold_docs(cls, nlp, tuples, gold_preproc, max_length=None,
                       noise_level=0.0):
        for raw_text, paragraph_tuples in tuples:
            if gold_preproc:
                raw_text = None
            else:
                paragraph_tuples = merge_sents(paragraph_tuples)
            docs = cls._make_docs(nlp, raw_text, paragraph_tuples,
                                  gold_preproc, noise_level=noise_level)
            golds = cls._make_golds(docs, paragraph_tuples)
            for doc, gold in zip(docs, golds):
                if (not max_length) or len(doc) < max_length:
                    yield doc, gold

    @classmethod
    def _make_docs(cls, nlp, raw_text, paragraph_tuples, gold_preproc,
                   noise_level=0.0):
        if raw_text is not None:
            raw_text = add_noise(raw_text, noise_level)
            return [nlp.make_doc(raw_text)]
        else:
            return [Doc(nlp.vocab,
                        words=add_noise(sent_tuples[1], noise_level))
                    for (sent_tuples, brackets) in paragraph_tuples]

    @classmethod
    def _make_golds(cls, docs, paragraph_tuples):
        if len(docs) != len(paragraph_tuples):
            raise ValueError(Errors.E070.format(n_docs=len(docs),
                                                n_annots=len(paragraph_tuples)))
        if len(docs) == 1:
            return [GoldParse.from_annot_tuples(docs[0],
                                                paragraph_tuples[0][0])]
        else:
            return [GoldParse.from_annot_tuples(doc, sent_tuples)
                    for doc, (sent_tuples, brackets)
                    in zip(docs, paragraph_tuples)]

    @staticmethod
    def walk_corpus(path):
        if not path.is_dir():
            return [path]
        paths = [path]
        locs = []
        seen = set()
        for path in paths:
            if str(path) in seen:
                continue
            seen.add(str(path))
            if path.parts[-1].startswith('.'):
                continue
            elif path.is_dir():
                paths.extend(path.iterdir())
            elif path.parts[-1].endswith('.json'):
                locs.append(path)
        return locs


def add_noise(orig, noise_level):
    if random.random() >= noise_level:
        return orig
    elif type(orig) == list:
        corrupted = [_corrupt(word, noise_level) for word in orig]
        corrupted = [w for w in corrupted if w]
        return corrupted
    else:
        return ''.join(_corrupt(c, noise_level) for c in orig)


def _corrupt(c, noise_level):
    if random.random() >= noise_level:
        return c
    elif c == ' ':
        return '\n'
    elif c == '\n':
        return ' '
    elif c in ['.', "'", "!", "?"]:
        return ''
    else:
        return c.lower()


def read_json_file(loc, docs_filter=None, limit=None):
    loc = util.ensure_path(loc)
    if loc.is_dir():
        for filename in loc.iterdir():
            yield from read_json_file(loc / filename, limit=limit)
    else:
        with loc.open('r', encoding='utf8') as file_:
            docs = ujson.load(file_)
        if limit is not None:
            docs = docs[:limit]
        for doc in docs:
            if docs_filter is not None and not docs_filter(doc):
                continue
            paragraphs = []
            for paragraph in doc['paragraphs']:
                sents = []
                for sent in paragraph['sentences']:
                    words = []
                    ids = []
                    tags = []
                    heads = []
                    labels = []
                    ner = []
                    for i, token in enumerate(sent['tokens']):
                        words.append(token['orth'])
                        ids.append(i)
                        tags.append(token.get('tag', '-'))
                        heads.append(token.get('head', 0) + i)
                        labels.append(token.get('dep', ''))
                        # Ensure ROOT label is case-insensitive
                        if labels[-1].lower() == 'root':
                            labels[-1] = 'ROOT'
                        ner.append(token.get('ner', '-'))
                    sents.append([
                        [ids, words, tags, heads, labels, ner],
                        sent.get('brackets', [])])
                if sents:
                    yield [paragraph.get('raw', None), sents]


def iob_to_biluo(tags):
    out = []
    curr_label = None
    tags = list(tags)
    while tags:
        out.extend(_consume_os(tags))
        out.extend(_consume_ent(tags))
    return out


def _consume_os(tags):
    while tags and tags[0] == 'O':
        yield tags.pop(0)


def _consume_ent(tags):
    if not tags:
        return []
    target = tags.pop(0).replace('B', 'I')
    length = 1
    while tags and tags[0] == target:
        length += 1
        tags.pop(0)
    label = target[2:]
    if length == 1:
        return ['U-' + label]
    else:
        start = 'B-' + label
        end = 'L-' + label
        middle = ['I-%s' % label for _ in range(1, length - 1)]
        return [start] + middle + [end]


cdef class GoldParse:
    """Collection for training annotations."""
    @classmethod
    def from_annot_tuples(cls, doc, annot_tuples, make_projective=False):
        _, words, tags, heads, deps, entities = annot_tuples
        return cls(doc, words=words, tags=tags, heads=heads, deps=deps,
                   entities=entities, make_projective=make_projective)

    def __init__(self, doc, annot_tuples=None, words=None, tags=None,
                 heads=None, deps=None, entities=None, make_projective=False,
                 cats=None):
        """Create a GoldParse.

        doc (Doc): The document the annotations refer to.
        words (iterable): A sequence of unicode word strings.
        tags (iterable): A sequence of strings, representing tag annotations.
        heads (iterable): A sequence of integers, representing syntactic
            head offsets.
        deps (iterable): A sequence of strings, representing the syntactic
            relation types.
        entities (iterable): A sequence of named entity annotations, either as
            BILUO tag strings, or as `(start_char, end_char, label)` tuples,
            representing the entity positions.
        cats (dict): Labels for text classification. Each key in the dictionary
            may be a string or an int, or a `(start_char, end_char, label)`
            tuple, indicating that the label is applied to only part of the
            document (usually a sentence). Unlike entity annotations, label
            annotations can overlap, i.e. a single word can be covered by
            multiple labelled spans. The TextCategorizer component expects
            true examples of a label to have the value 1.0, and negative
            examples of a label to have the value 0.0. Labels not in the
            dictionary are treated as missing - the gradient for those labels
            will be zero.
        RETURNS (GoldParse): The newly constructed object.
        """
        if words is None:
            words = [token.text for token in doc]
        if tags is None:
            tags = [None for _ in doc]
        if heads is None:
            heads = [None for token in doc]
        if deps is None:
            deps = [None for _ in doc]
        if entities is None:
            entities = [None for _ in doc]
        elif len(entities) == 0:
            entities = ['O' for _ in doc]
        elif not isinstance(entities[0], basestring):
            # Assume we have entities specified by character offset.
            entities = biluo_tags_from_offsets(doc, entities)

        self.mem = Pool()
        self.loss = 0
        self.length = len(doc)

        # These are filled by the tagger/parser/entity recogniser
        self.c.tags = <int*>self.mem.alloc(len(doc), sizeof(int))
        self.c.heads = <int*>self.mem.alloc(len(doc), sizeof(int))
        self.c.labels = <attr_t*>self.mem.alloc(len(doc), sizeof(attr_t))
        self.c.has_dep = <int*>self.mem.alloc(len(doc), sizeof(int))
        self.c.sent_start = <int*>self.mem.alloc(len(doc), sizeof(int))
        self.c.ner = <Transition*>self.mem.alloc(len(doc), sizeof(Transition))

        self.cats = {} if cats is None else dict(cats)
        self.words = [None] * len(doc)
        self.tags = [None] * len(doc)
        self.heads = [None] * len(doc)
        self.labels = [None] * len(doc)
        self.ner = [None] * len(doc)

        self.cand_to_gold = align([t.orth_ for t in doc], words)
        self.gold_to_cand = align(words, [t.orth_ for t in doc])

        annot_tuples = (range(len(words)), words, tags, heads, deps, entities)
        self.orig_annot = list(zip(*annot_tuples))

        for i, gold_i in enumerate(self.cand_to_gold):
            if doc[i].text.isspace():
                self.words[i] = doc[i].text
                self.tags[i] = 'SP'
                self.heads[i] = None
                self.labels[i] = None
                self.ner[i] = 'O'
            if gold_i is None:
                pass
            else:
                self.words[i] = words[gold_i]
                self.tags[i] = tags[gold_i]
                if heads[gold_i] is None:
                    self.heads[i] = None
                else:
                    self.heads[i] = self.gold_to_cand[heads[gold_i]]
                self.labels[i] = deps[gold_i]
                self.ner[i] = entities[gold_i]

        cycle = nonproj.contains_cycle(self.heads)
        if cycle is not None:
            raise ValueError(Errors.E069.format(cycle=cycle))

        if make_projective:
            proj_heads, _ = nonproj.projectivize(self.heads, self.labels)
            self.heads = proj_heads

    def __len__(self):
        """Get the number of gold-standard tokens.

        RETURNS (int): The number of gold-standard tokens.
        """
        return self.length

    @property
    def is_projective(self):
        """Whether the provided syntactic annotations form a projective
        dependency tree.
        """
        return not nonproj.is_nonproj_tree(self.heads)

    @property
    def sent_starts(self):
        return [self.c.sent_start[i] for i in range(self.length)]


def biluo_tags_from_offsets(doc, entities, missing='O'):
    """Encode labelled spans into per-token tags, using the
    Begin/In/Last/Unit/Out scheme (BILUO).

    doc (Doc): The document that the entity offsets refer to. The output tags
        will refer to the token boundaries within the document.
    entities (iterable): A sequence of `(start, end, label)` triples. `start`
        and `end` should be character-offset integers denoting the slice into
        the original string.
    RETURNS (list): A list of unicode strings, describing the tags. Each tag
        string will be of the form either "", "O" or "{action}-{label}", where
        action is one of "B", "I", "L", "U". The string "-" is used where the
        entity offsets don't align with the tokenization in the `Doc` object.
        The training algorithm will view these as missing values. "O" denotes a
        non-entity token. "B" denotes the beginning of a multi-token entity,
        "I" the inside of an entity of three or more tokens, and "L" the end
        of an entity of two or more tokens. "U" denotes a single-token entity.

    EXAMPLE:
        >>> text = 'I like London.'
        >>> entities = [(len('I like '), len('I like London'), 'LOC')]
        >>> doc = nlp.tokenizer(text)
        >>> tags = biluo_tags_from_offsets(doc, entities)
        >>> assert tags == ['O', 'O', 'U-LOC', 'O']
    """
    starts = {token.idx: token.i for token in doc}
    ends = {token.idx+len(token): token.i for token in doc}
    biluo = ['-' for _ in doc]
    # Handle entity cases
    for start_char, end_char, label in entities:
        start_token = starts.get(start_char)
        end_token = ends.get(end_char)
        # Only interested if the tokenization is correct
        if start_token is not None and end_token is not None:
            if start_token == end_token:
                biluo[start_token] = 'U-%s' % label
            else:
                biluo[start_token] = 'B-%s' % label
                for i in range(start_token+1, end_token):
                    biluo[i] = 'I-%s' % label
                biluo[end_token] = 'L-%s' % label
    # Now distinguish the O cases from ones where we miss the tokenization
    entity_chars = set()
    for start_char, end_char, label in entities:
        for i in range(start_char, end_char):
            entity_chars.add(i)
    for token in doc:
        for i in range(token.idx, token.idx+len(token)):
            if i in entity_chars:
                break
        else:
            biluo[token.i] = missing
    return biluo


def offsets_from_biluo_tags(doc, tags):
    """Encode per-token tags following the BILUO scheme into entity offsets.

    doc (Doc): The document that the BILUO tags refer to.
    entities (iterable): A sequence of BILUO tags with each tag describing one
        token. Each tags string will be of the form of either "", "O" or
        "{action}-{label}", where action is one of "B", "I", "L", "U".
    RETURNS (list): A sequence of `(start, end, label)` triples. `start` and
        `end` will be character-offset integers denoting the slice into the
        original string.
    """
    token_offsets = tags_to_entities(tags)
    offsets = []
    for label, start_idx, end_idx in token_offsets:
        span = doc[start_idx : end_idx + 1]
        offsets.append((span.start_char, span.end_char, label))
    return offsets


def is_punct_label(label):
    return label == 'P' or label.lower() == 'punct'



# coding: utf8
from __future__ import unicode_literals

from ...symbols import ORTH, LEMMA, TAG, NORM, PRON_LEMMA


_exc = {
    "auf'm": [
        {ORTH: "auf", LEMMA: "auf"},
        {ORTH: "'m", LEMMA: "der", NORM: "dem"}],

    "du's": [
        {ORTH: "du", LEMMA: PRON_LEMMA, TAG: "PPER"},
        {ORTH: "'s", LEMMA: PRON_LEMMA, TAG: "PPER", NORM: "es"}],

    "er's": [
        {ORTH: "er", LEMMA: PRON_LEMMA, TAG: "PPER"},
        {ORTH: "'s", LEMMA: PRON_LEMMA, TAG: "PPER", NORM: "es"}],

    "hinter'm": [
        {ORTH: "hinter", LEMMA: "hinter"},
        {ORTH: "'m", LEMMA: "der", NORM: "dem"}],

    "ich's": [
        {ORTH: "ich", LEMMA: PRON_LEMMA, TAG: "PPER"},
        {ORTH: "'s", LEMMA: PRON_LEMMA, TAG: "PPER", NORM: "es"}],

    "ihr's": [
        {ORTH: "ihr", LEMMA: PRON_LEMMA, TAG: "PPER"},
        {ORTH: "'s", LEMMA: PRON_LEMMA, TAG: "PPER", NORM: "es"}],

    "sie's": [
        {ORTH: "sie", LEMMA: PRON_LEMMA, TAG: "PPER"},
        {ORTH: "'s", LEMMA: PRON_LEMMA, TAG: "PPER", NORM: "es"}],

    "unter'm": [
        {ORTH: "unter", LEMMA: "unter"},
        {ORTH: "'m", LEMMA: "der", NORM: "dem"}],

    "vor'm": [
        {ORTH: "vor", LEMMA: "vor"},
        {ORTH: "'m", LEMMA: "der", NORM: "dem"}],

    "wir's": [
        {ORTH: "wir", LEMMA: PRON_LEMMA, TAG: "PPER"},
        {ORTH: "'s", LEMMA: PRON_LEMMA, TAG: "PPER", NORM: "es"}],

    "über'm": [
        {ORTH: "über", LEMMA: "über"},
        {ORTH: "'m", LEMMA: "der", NORM: "dem"}]
}


for exc_data in [
    {ORTH: "'S", LEMMA: PRON_LEMMA, NORM: "'s", TAG: "PPER"},
    {ORTH: "'s", LEMMA: PRON_LEMMA, NORM: "'s", TAG: "PPER"},
    {ORTH: "S'", LEMMA: PRON_LEMMA, NORM: "'s", TAG: "PPER"},
    {ORTH: "s'", LEMMA: PRON_LEMMA, NORM: "'s", TAG: "PPER"},
    {ORTH: "'n", LEMMA: "ein", NORM: "ein"},
    {ORTH: "'ne", LEMMA: "eine", NORM: "eine"},
    {ORTH: "'nen", LEMMA: "ein", NORM: "einen"},
    {ORTH: "'nem", LEMMA: "ein", NORM: "einem"},
    {ORTH: "Abb.", LEMMA: "Abbildung", NORM: "Abbildung"},
    {ORTH: "Abk.", LEMMA: "Abkürzung", NORM: "Abkürzung"},
    {ORTH: "Abt.", LEMMA: "Abteilung", NORM: "Abteilung"},
    {ORTH: "Apr.", LEMMA: "April", NORM: "April"},
    {ORTH: "Aug.", LEMMA: "August", NORM: "August"},
    {ORTH: "Bd.", LEMMA: "Band", NORM: "Band"},
    {ORTH: "Betr.", LEMMA: "Betreff", NORM: "Betreff"},
    {ORTH: "Bf.", LEMMA: "Bahnhof", NORM: "Bahnhof"},
    {ORTH: "Bhf.", LEMMA: "Bahnhof", NORM: "Bahnhof"},
    {ORTH: "Bsp.", LEMMA: "Beispiel", NORM: "Beispiel"},
    {ORTH: "Dez.", LEMMA: "Dezember", NORM: "Dezember"},
    {ORTH: "Di.", LEMMA: "Dienstag", NORM: "Dienstag"},
    {ORTH: "Do.", LEMMA: "Donnerstag", NORM: "Donnerstag"},
    {ORTH: "Fa.", LEMMA: "Firma", NORM: "Firma"},
    {ORTH: "Fam.", LEMMA: "Familie", NORM: "Familie"},
    {ORTH: "Feb.", LEMMA: "Februar", NORM: "Februar"},
    {ORTH: "Fr.", LEMMA: "Frau", NORM: "Frau"},
    {ORTH: "Frl.", LEMMA: "Fräulein", NORM: "Fräulein"},
    {ORTH: "Hbf.", LEMMA: "Hauptbahnhof", NORM: "Hauptbahnhof"},
    {ORTH: "Hr.", LEMMA: "Herr", NORM: "Herr"},
    {ORTH: "Hrn.", LEMMA: "Herr", NORM: "Herrn"},
    {ORTH: "Jan.", LEMMA: "Januar", NORM: "Januar"},
    {ORTH: "Jh.", LEMMA: "Jahrhundert", NORM: "Jahrhundert"},
    {ORTH: "Jhd.", LEMMA: "Jahrhundert", NORM: "Jahrhundert"},
    {ORTH: "Jul.", LEMMA: "Juli", NORM: "Juli"},
    {ORTH: "Jun.", LEMMA: "Juni", NORM: "Juni"},
    {ORTH: "Mi.", LEMMA: "Mittwoch", NORM: "Mittwoch"},
    {ORTH: "Mio.", LEMMA: "Million", NORM: "Million"},
    {ORTH: "Mo.", LEMMA: "Montag", NORM: "Montag"},
    {ORTH: "Mrd.", LEMMA: "Milliarde", NORM: "Milliarde"},
    {ORTH: "Mrz.", LEMMA: "März", NORM: "März"},
    {ORTH: "MwSt.", LEMMA: "Mehrwertsteuer", NORM: "Mehrwertsteuer"},
    {ORTH: "Mär.", LEMMA: "März", NORM: "März"},
    {ORTH: "Nov.", LEMMA: "November", NORM: "November"},
    {ORTH: "Nr.", LEMMA: "Nummer", NORM: "Nummer"},
    {ORTH: "Okt.", LEMMA: "Oktober", NORM: "Oktober"},
    {ORTH: "Orig.", LEMMA: "Original", NORM: "Original"},
    {ORTH: "Pkt.", LEMMA: "Punkt", NORM: "Punkt"},
    {ORTH: "Prof.", LEMMA: "Professor", NORM: "Professor"},
    {ORTH: "Red.", LEMMA: "Redaktion", NORM: "Redaktion"},
    {ORTH: "Sa.", LEMMA: "Samstag", NORM: "Samstag"},
    {ORTH: "Sep.", LEMMA: "September", NORM: "September"},
    {ORTH: "Sept.", LEMMA: "September", NORM: "September"},
    {ORTH: "So.", LEMMA: "Sonntag", NORM: "Sonntag"},
    {ORTH: "Std.", LEMMA: "Stunde", NORM: "Stunde"},
    {ORTH: "Str.", LEMMA: "Straße", NORM: "Straße"},
    {ORTH: "Tel.", LEMMA: "Telefon", NORM: "Telefon"},
    {ORTH: "Tsd.", LEMMA: "Tausend", NORM: "Tausend"},
    {ORTH: "Univ.", LEMMA: "Universität", NORM: "Universität"},
    {ORTH: "abzgl.", LEMMA: "abzüglich", NORM: "abzüglich"},
    {ORTH: "allg.", LEMMA: "allgemein", NORM: "allgemein"},
    {ORTH: "bspw.", LEMMA: "beispielsweise", NORM: "beispielsweise"},
    {ORTH: "bzgl.", LEMMA: "bezüglich", NORM: "bezüglich"},
    {ORTH: "bzw.", LEMMA: "beziehungsweise", NORM: "beziehungsweise"},
    {ORTH: "d.h.", LEMMA: "das heißt"},
    {ORTH: "dgl.", LEMMA: "dergleichen", NORM: "dergleichen"},
    {ORTH: "ebd.", LEMMA: "ebenda", NORM: "ebenda"},
    {ORTH: "eigtl.", LEMMA: "eigentlich", NORM: "eigentlich"},
    {ORTH: "engl.", LEMMA: "englisch", NORM: "englisch"},
    {ORTH: "evtl.", LEMMA: "eventuell", NORM: "eventuell"},
    {ORTH: "frz.", LEMMA: "französisch", NORM: "französisch"},
    {ORTH: "gegr.", LEMMA: "gegründet", NORM: "gegründet"},
    {ORTH: "ggf.", LEMMA: "gegebenenfalls", NORM: "gegebenenfalls"},
    {ORTH: "ggfs.", LEMMA: "gegebenenfalls", NORM: "gegebenenfalls"},
    {ORTH: "ggü.", LEMMA: "gegenüber", NORM: "gegenüber"},
    {ORTH: "i.O.", LEMMA: "in Ordnung"},
    {ORTH: "i.d.R.", LEMMA: "in der Regel"},
    {ORTH: "incl.", LEMMA: "inklusive", NORM: "inklusive"},
    {ORTH: "inkl.", LEMMA: "inklusive", NORM: "inklusive"},
    {ORTH: "insb.", LEMMA: "insbesondere", NORM: "insbesondere"},
    {ORTH: "kath.", LEMMA: "katholisch", NORM: "katholisch"},
    {ORTH: "lt.", LEMMA: "laut", NORM: "laut"},
    {ORTH: "max.", LEMMA: "maximal", NORM: "maximal"},
    {ORTH: "min.", LEMMA: "minimal", NORM: "minimal"},
    {ORTH: "mind.", LEMMA: "mindestens", NORM: "mindestens"},
    {ORTH: "mtl.", LEMMA: "monatlich", NORM: "monatlich"},
    {ORTH: "n.Chr.", LEMMA: "nach Christus"},
    {ORTH: "orig.", LEMMA: "original", NORM: "original"},
    {ORTH: "röm.", LEMMA: "römisch", NORM: "römisch"},
    {ORTH: "s.o.", LEMMA: "siehe oben"},
    {ORTH: "sog.", LEMMA: "so genannt"},
    {ORTH: "stellv.", LEMMA: "stellvertretend"},
    {ORTH: "tägl.", LEMMA: "täglich", NORM: "täglich"},
    {ORTH: "u.U.", LEMMA: "unter Umständen"},
    {ORTH: "u.s.w.", LEMMA: "und so weiter"},
    {ORTH: "u.v.m.", LEMMA: "und vieles mehr"},
    {ORTH: "usf.", LEMMA: "und so fort"},
    {ORTH: "usw.", LEMMA: "und so weiter"},
    {ORTH: "uvm.", LEMMA: "und vieles mehr"},
    {ORTH: "v.Chr.", LEMMA: "vor Christus"},
    {ORTH: "v.a.", LEMMA: "vor allem"},
    {ORTH: "v.l.n.r.", LEMMA: "von links nach rechts"},
    {ORTH: "vgl.", LEMMA: "vergleiche", NORM: "vergleiche"},
    {ORTH: "vllt.", LEMMA: "vielleicht", NORM: "vielleicht"},
    {ORTH: "vlt.", LEMMA: "vielleicht", NORM: "vielleicht"},
    {ORTH: "z.B.", LEMMA: "zum Beispiel"},
    {ORTH: "z.Bsp.", LEMMA: "zum Beispiel"},
    {ORTH: "z.T.", LEMMA: "zum Teil"},
    {ORTH: "z.Z.", LEMMA: "zur Zeit"},
    {ORTH: "z.Zt.", LEMMA: "zur Zeit"},
    {ORTH: "z.b.", LEMMA: "zum Beispiel"},
    {ORTH: "zzgl.", LEMMA: "zuzüglich"},
    {ORTH: "österr.", LEMMA: "österreichisch", NORM: "österreichisch"}]:
    _exc[exc_data[ORTH]] = [exc_data]


for orth in [
    "A.C.", "a.D.", "A.D.", "A.G.", "a.M.", "a.Z.", "Abs.", "adv.", "al.",
    "B.A.", "B.Sc.", "betr.", "biol.", "Biol.", "ca.", "Chr.", "Cie.", "co.",
    "Co.", "D.C.", "Dipl.-Ing.", "Dipl.", "Dr.", "e.g.", "e.V.", "ehem.",
    "entspr.", "erm.", "etc.", "ev.", "G.m.b.H.", "geb.", "Gebr.", "gem.",
    "h.c.", "Hg.", "hrsg.", "Hrsg.", "i.A.", "i.e.", "i.G.", "i.Tr.", "i.V.",
    "Ing.", "jr.", "Jr.", "jun.", "jur.", "K.O.", "L.A.", "lat.", "M.A.",
    "m.E.", "m.M.", "M.Sc.", "Mr.", "N.Y.", "N.Y.C.", "nat.", "o.a.",
    "o.ä.", "o.g.", "o.k.", "O.K.", "p.a.", "p.s.", "P.S.", "pers.", "phil.",
    "q.e.d.", "R.I.P.", "rer.", "sen.", "St.", "std.", "u.a.", "U.S.", "U.S.A.",
    "U.S.S.", "Vol.", "vs.", "wiss."]:
    _exc[orth] = [{ORTH: orth}]


TOKENIZER_EXCEPTIONS = _exc



from __future__ import unicode_literals
import pytest
from ...util import load_model

@pytest.mark.models("en_core_web_md")
@pytest.mark.models("es_core_news_md")
def test_models_with_different_vectors():
    nlp = load_model('en_core_web_md')
    doc = nlp(u'hello world')
    nlp2 = load_model('es_core_news_md')
    doc2 = nlp2(u'hola')
    doc = nlp(u'hello world')



# coding: utf8
# cython: profile=True
from __future__ import unicode_literals

import numpy
import dill

from collections import OrderedDict
from thinc.neural.util import get_array_module
from .lexeme cimport EMPTY_LEXEME
from .lexeme cimport Lexeme
from .strings cimport hash_string
from .typedefs cimport attr_t
from .tokens.token cimport Token
from .attrs cimport PROB, LANG, ORTH, TAG
from .structs cimport SerializedLexemeC

from .compat import copy_reg, basestring_
from .errors import Errors
from .lemmatizer import Lemmatizer
from .attrs import intify_attrs
from .vectors import Vectors
from ._ml import link_vectors_to_models
from . import util


cdef class Vocab:
    """A look-up table that allows you to access `Lexeme` objects. The `Vocab`
    instance also provides access to the `StringStore`, and owns underlying
    C-data that is shared between `Doc` objects.
    """
    def __init__(self, lex_attr_getters=None, tag_map=None, lemmatizer=None,
                 strings=tuple(), oov_prob=-20., **deprecated_kwargs):
        """Create the vocabulary.

        lex_attr_getters (dict): A dictionary mapping attribute IDs to
            functions to compute them. Defaults to `None`.
        tag_map (dict): Dictionary mapping fine-grained tags to coarse-grained
            parts-of-speech, and optionally morphological attributes.
        lemmatizer (object): A lemmatizer. Defaults to `None`.
        strings (StringStore): StringStore that maps strings to integers, and
            vice versa.
        RETURNS (Vocab): The newly constructed object.
        """
        lex_attr_getters = lex_attr_getters if lex_attr_getters is not None else {}
        tag_map = tag_map if tag_map is not None else {}
        if lemmatizer in (None, True, False):
            lemmatizer = Lemmatizer({}, {}, {})
        self.cfg = {'oov_prob': oov_prob}
        self.mem = Pool()
        self._by_hash = PreshMap()
        self._by_orth = PreshMap()
        self.strings = StringStore()
        self.length = 0
        if strings:
            for string in strings:
                _ = self[string]
        self.lex_attr_getters = lex_attr_getters
        self.morphology = Morphology(self.strings, tag_map, lemmatizer)
        self.vectors = Vectors()

    property lang:
        def __get__(self):
            langfunc = None
            if self.lex_attr_getters:
                langfunc = self.lex_attr_getters.get(LANG, None)
            return langfunc('_') if langfunc else ''

    def __len__(self):
        """The current number of lexemes stored.

        RETURNS (int): The current number of lexemes stored.
        """
        return self.length

    def add_flag(self, flag_getter, int flag_id=-1):
        """Set a new boolean flag to words in the vocabulary.

        The flag_getter function will be called over the words currently in the
        vocab, and then applied to new words as they occur. You'll then be able
        to access the flag value on each token using token.check_flag(flag_id).
        See also: `Lexeme.set_flag`, `Lexeme.check_flag`, `Token.set_flag`,
        `Token.check_flag`.

        flag_getter (callable): A function `f(unicode) -> bool`, to get the
            flag value.
        flag_id (int): An integer between 1 and 63 (inclusive), specifying
            the bit at which the flag will be stored. If -1, the lowest
            available bit will be chosen.
        RETURNS (int): The integer ID by which the flag value can be checked.

        EXAMPLE:
            >>> my_product_getter = lambda text: text in ['spaCy', 'dislaCy']
            >>> MY_PRODUCT = nlp.vocab.add_flag(my_product_getter)
            >>> doc = nlp(u'I like spaCy')
            >>> assert doc[2].check_flag(MY_PRODUCT) == True
        """
        if flag_id == -1:
            for bit in range(1, 64):
                if bit not in self.lex_attr_getters:
                    flag_id = bit
                    break
            else:
                raise ValueError(Errors.E062)
        elif flag_id >= 64 or flag_id < 1:
            raise ValueError(Errors.E063.format(value=flag_id))
        for lex in self:
            lex.set_flag(flag_id, flag_getter(lex.orth_))
        self.lex_attr_getters[flag_id] = flag_getter
        return flag_id

    cdef const LexemeC* get(self, Pool mem, unicode string) except NULL:
        """Get a pointer to a `LexemeC` from the lexicon, creating a new
        `Lexeme` if necessary using memory acquired from the given pool. If the
        pool is the lexicon's own memory, the lexeme is saved in the lexicon.
        """
        if string == u'':
            return &EMPTY_LEXEME
        cdef LexemeC* lex
        cdef hash_t key = hash_string(string)
        lex = <LexemeC*>self._by_hash.get(key)
        cdef size_t addr
        if lex != NULL:
            if lex.orth != self.strings[string]:
                raise KeyError(Errors.E064.format(string=lex.orth,
                                                  orth=self.strings[string],
                                                  orth_id=string))
            return lex
        else:
            return self._new_lexeme(mem, string)

    cdef const LexemeC* get_by_orth(self, Pool mem, attr_t orth) except NULL:
        """Get a pointer to a `LexemeC` from the lexicon, creating a new
        `Lexeme` if necessary using memory acquired from the given pool. If the
        pool is the lexicon's own memory, the lexeme is saved in the lexicon.
        """
        if orth == 0:
            return &EMPTY_LEXEME
        cdef LexemeC* lex
        lex = <LexemeC*>self._by_orth.get(orth)
        if lex != NULL:
            return lex
        else:
            return self._new_lexeme(mem, self.strings[orth])

    cdef const LexemeC* _new_lexeme(self, Pool mem, unicode string) except NULL:
        cdef hash_t key
        if len(string) < 3 or self.length < 10000:
            mem = self.mem
        cdef bint is_oov = mem is not self.mem
        lex = <LexemeC*>mem.alloc(sizeof(LexemeC), 1)
        lex.orth = self.strings.add(string)
        lex.length = len(string)
        if self.vectors is not None:
            lex.id = self.vectors.key2row.get(lex.orth, 0)
        else:
            lex.id = 0
        if self.lex_attr_getters is not None:
            for attr, func in self.lex_attr_getters.items():
                value = func(string)
                if isinstance(value, unicode):
                    value = self.strings.add(value)
                if attr == PROB:
                    lex.prob = value
                elif value is not None:
                    Lexeme.set_struct_attr(lex, attr, value)
        if not is_oov:
            key = hash_string(string)
            self._add_lex_to_vocab(key, lex)
        if lex == NULL:
            raise ValueError(Errors.E085.format(string=string))
        return lex

    cdef int _add_lex_to_vocab(self, hash_t key, const LexemeC* lex) except -1:
        self._by_hash.set(key, <void*>lex)
        self._by_orth.set(lex.orth, <void*>lex)
        self.length += 1

    def __contains__(self, key):
        """Check whether the string or int key has an entry in the vocabulary.

        string (unicode): The ID string.
        RETURNS (bool) Whether the string has an entry in the vocabulary.
        """
        cdef hash_t int_key
        if isinstance(key, bytes):
            int_key = hash_string(key.decode('utf8'))
        elif isinstance(key, unicode):
            int_key = hash_string(key)
        else:
            int_key = key
        lex = self._by_hash.get(int_key)
        return lex is not NULL

    def __iter__(self):
        """Iterate over the lexemes in the vocabulary.

        YIELDS (Lexeme): An entry in the vocabulary.
        """
        cdef attr_t key
        cdef size_t addr
        for key, addr in self._by_orth.items():
            lex = Lexeme(self, key)
            yield lex

    def __getitem__(self, id_or_string):
        """Retrieve a lexeme, given an int ID or a unicode string. If a
        previously unseen unicode string is given, a new lexeme is created and
        stored.

        id_or_string (int or unicode): The integer ID of a word, or its unicode
            string. If `int >= Lexicon.size`, `IndexError` is raised. If
            `id_or_string` is neither an int nor a unicode string, `ValueError`
            is raised.
        RETURNS (Lexeme): The lexeme indicated by the given ID.

        EXAMPLE:
            >>> apple = nlp.vocab.strings['apple']
            >>> assert nlp.vocab[apple] == nlp.vocab[u'apple']
        """
        cdef attr_t orth
        if isinstance(id_or_string, unicode):
            orth = self.strings.add(id_or_string)
        else:
            orth = id_or_string
        return Lexeme(self, orth)

    cdef const TokenC* make_fused_token(self, substrings) except NULL:
        cdef int i
        tokens = <TokenC*>self.mem.alloc(len(substrings) + 1, sizeof(TokenC))
        for i, props in enumerate(substrings):
            props = intify_attrs(props, strings_map=self.strings,
                                 _do_deprecated=True)
            token = &tokens[i]
            # Set the special tokens up to have arbitrary attributes
            lex = <LexemeC*>self.get_by_orth(self.mem, props[ORTH])
            token.lex = lex
            if TAG in props:
                self.morphology.assign_tag(token, props[TAG])
            for attr_id, value in props.items():
                Token.set_struct_attr(token, attr_id, value)
                Lexeme.set_struct_attr(lex, attr_id, value)
        return tokens

    @property
    def vectors_length(self):
        return self.vectors.data.shape[1]

    def reset_vectors(self, *, width=None, shape=None):
        """Drop the current vector table. Because all vectors must be the same
        width, you have to call this to change the size of the vectors.
        """
        if width is not None and shape is not None:
            raise ValueError(Errors.E065.format(width=width, shape=shape))
        elif shape is not None:
            self.vectors = Vectors(shape=shape)
        else:
            width = width if width is not None else self.vectors.data.shape[1]
            self.vectors = Vectors(shape=(self.vectors.shape[0], width))

    def prune_vectors(self, nr_row, batch_size=1024):
        """Reduce the current vector table to `nr_row` unique entries. Words
        mapped to the discarded vectors will be remapped to the closest vector
        among those remaining.

        For example, suppose the original table had vectors for the words:
        ['sat', 'cat', 'feline', 'reclined']. If we prune the vector table to,
        two rows, we would discard the vectors for 'feline' and 'reclined'.
        These words would then be remapped to the closest remaining vector
        -- so "feline" would have the same vector as "cat", and "reclined"
        would have the same vector as "sat".

        The similarities are judged by cosine. The original vectors may
        be large, so the cosines are calculated in minibatches, to reduce
        memory usage.

        nr_row (int): The number of rows to keep in the vector table.
        batch_size (int): Batch of vectors for calculating the similarities.
            Larger batch sizes might be faster, while temporarily requiring
            more memory.
        RETURNS (dict): A dictionary keyed by removed words mapped to
            `(string, score)` tuples, where `string` is the entry the removed
            word was mapped to, and `score` the similarity score between the
            two words.
        """
        xp = get_array_module(self.vectors.data)
        # Make prob negative so it sorts by rank ascending
        # (key2row contains the rank)
        priority = [(-lex.prob, self.vectors.key2row[lex.orth], lex.orth)
                    for lex in self if lex.orth in self.vectors.key2row]
        priority.sort()
        indices = xp.asarray([i for (prob, i, key) in priority], dtype='i')
        keys = xp.asarray([key for (prob, i, key) in priority], dtype='uint64')

        keep = xp.ascontiguousarray(self.vectors.data[indices[:nr_row]])
        toss = xp.ascontiguousarray(self.vectors.data[indices[nr_row:]])

        self.vectors = Vectors(data=keep, keys=keys)

        syn_keys, syn_rows, scores = self.vectors.most_similar(toss)

        remap = {}
        for i, key in enumerate(keys[nr_row:]):
            self.vectors.add(key, row=syn_rows[i])
            word = self.strings[key]
            synonym = self.strings[syn_keys[i]]
            score = scores[i]
            remap[word] = (synonym, score)
        link_vectors_to_models(self)
        return remap

    def get_vector(self, orth):
        """Retrieve a vector for a word in the vocabulary. Words can be looked
        up by string or int ID. If no vectors data is loaded, ValueError is
        raised.

        RETURNS (numpy.ndarray): A word vector. Size
            and shape determined by the `vocab.vectors` instance. Usually, a
            numpy ndarray of shape (300,) and dtype float32.
        """
        if isinstance(orth, basestring_):
            orth = self.strings.add(orth)
        if orth in self.vectors.key2row:
            return self.vectors[orth]
        else:
            return numpy.zeros((self.vectors_length,), dtype='f')

    def set_vector(self, orth, vector):
        """Set a vector for a word in the vocabulary. Words can be referenced
        by string or int ID.
        """
        if isinstance(orth, basestring_):
            orth = self.strings.add(orth)
        if self.vectors.is_full and orth not in self.vectors:
            new_rows = max(100, int(self.vectors.shape[0]*1.3))
            if self.vectors.shape[1] == 0:
                width = vector.size
            else:
                width = self.vectors.shape[1]
            self.vectors.resize((new_rows, width))
            lex = self[orth] # Adds worse to vocab
            self.vectors.add(orth, vector=vector)
        self.vectors.add(orth, vector=vector)

    def has_vector(self, orth):
        """Check whether a word has a vector. Returns False if no vectors have
        been loaded. Words can be looked up by string or int ID."""
        if isinstance(orth, basestring_):
            orth = self.strings.add(orth)
        return orth in self.vectors

    def to_disk(self, path, **exclude):
        """Save the current state to a directory.

        path (unicode or Path): A path to a directory, which will be created if
            it doesn't exist. Paths may be either strings or Path-like objects.
        """
        path = util.ensure_path(path)
        if not path.exists():
            path.mkdir()
        self.strings.to_disk(path / 'strings.json')
        with (path / 'lexemes.bin').open('wb') as file_:
            file_.write(self.lexemes_to_bytes())
        if self.vectors is not None:
            self.vectors.to_disk(path)

    def from_disk(self, path, **exclude):
        """Loads state from a directory. Modifies the object in place and
        returns it.

        path (unicode or Path): A path to a directory. Paths may be either
            strings or `Path`-like objects.
        RETURNS (Vocab): The modified `Vocab` object.
        """
        path = util.ensure_path(path)
        self.strings.from_disk(path / 'strings.json')
        with (path / 'lexemes.bin').open('rb') as file_:
            self.lexemes_from_bytes(file_.read())
        if self.vectors is not None:
            self.vectors.from_disk(path, exclude='strings.json')
        if self.vectors.name is not None:
            link_vectors_to_models(self)
        return self

    def to_bytes(self, **exclude):
        """Serialize the current state to a binary string.

        **exclude: Named attributes to prevent from being serialized.
        RETURNS (bytes): The serialized form of the `Vocab` object.
        """
        def deserialize_vectors():
            if self.vectors is None:
                return None
            else:
                return self.vectors.to_bytes()

        getters = OrderedDict((
            ('strings', lambda: self.strings.to_bytes()),
            ('lexemes', lambda: self.lexemes_to_bytes()),
            ('vectors', deserialize_vectors)
        ))
        return util.to_bytes(getters, exclude)

    def from_bytes(self, bytes_data, **exclude):
        """Load state from a binary string.

        bytes_data (bytes): The data to load from.
        **exclude: Named attributes to prevent from being loaded.
        RETURNS (Vocab): The `Vocab` object.
        """
        def serialize_vectors(b):
            if self.vectors is None:
                return None
            else:
                return self.vectors.from_bytes(b)
        setters = OrderedDict((
            ('strings', lambda b: self.strings.from_bytes(b)),
            ('lexemes', lambda b: self.lexemes_from_bytes(b)),
            ('vectors', lambda b: serialize_vectors(b))
        ))
        util.from_bytes(bytes_data, setters, exclude)
        if self.vectors.name is not None:
            link_vectors_to_models(self)
        return self

    def lexemes_to_bytes(self):
        cdef hash_t key
        cdef size_t addr
        cdef LexemeC* lexeme = NULL
        cdef SerializedLexemeC lex_data
        cdef int size = 0
        for key, addr in self._by_hash.items():
            if addr == 0:
                continue
            size += sizeof(lex_data.data)
        byte_string = b'\0' * size
        byte_ptr = <unsigned char*>byte_string
        cdef int j
        cdef int i = 0
        for key, addr in self._by_hash.items():
            if addr == 0:
                continue
            lexeme = <LexemeC*>addr
            lex_data = Lexeme.c_to_bytes(lexeme)
            for j in range(sizeof(lex_data.data)):
                byte_ptr[i] = lex_data.data[j]
                i += 1
        return byte_string

    def lexemes_from_bytes(self, bytes bytes_data):
        """Load the binary vocabulary data from the given string."""
        cdef LexemeC* lexeme
        cdef hash_t key
        cdef unicode py_str
        cdef int i = 0
        cdef int j = 0
        cdef SerializedLexemeC lex_data
        chunk_size = sizeof(lex_data.data)
        cdef void* ptr
        cdef unsigned char* bytes_ptr = bytes_data
        for i in range(0, len(bytes_data), chunk_size):
            lexeme = <LexemeC*>self.mem.alloc(1, sizeof(LexemeC))
            for j in range(sizeof(lex_data.data)):
                lex_data.data[j] = bytes_ptr[i+j]
            Lexeme.c_from_bytes(lexeme, lex_data)

            ptr = self.strings._map.get(lexeme.orth)
            if ptr == NULL:
                continue
            py_str = self.strings[lexeme.orth]
            if self.strings[py_str] != lexeme.orth:
                raise ValueError(Errors.E086.format(string=py_str,
                                                    orth_id=lexeme.orth,
                                                    hash_id=self.strings[py_str]))
            key = hash_string(py_str)
            self._by_hash.set(key, lexeme)
            self._by_orth.set(lexeme.orth, lexeme)
            self.length += 1

    def _reset_cache(self, keys, strings):
        for k in keys:
            del self._by_hash[k]

        if len(strings) != 0:
            self._by_orth = PreshMap()


def pickle_vocab(vocab):
    sstore = vocab.strings
    vectors = vocab.vectors
    morph = vocab.morphology
    length = vocab.length
    data_dir = vocab.data_dir
    lex_attr_getters = dill.dumps(vocab.lex_attr_getters)
    lexemes_data = vocab.lexemes_to_bytes()
    return (unpickle_vocab,
            (sstore, vectors, morph, data_dir, lex_attr_getters, lexemes_data, length))


def unpickle_vocab(sstore, vectors, morphology, data_dir,
                   lex_attr_getters, bytes lexemes_data, int length):
    cdef Vocab vocab = Vocab()
    vocab.length = length
    vocab.vectors = vectors
    vocab.strings = sstore
    vocab.morphology = morphology
    vocab.data_dir = data_dir
    vocab.lex_attr_getters = dill.loads(lex_attr_getters)
    vocab.lexemes_from_bytes(lexemes_data)
    vocab.length = length
    return vocab


copy_reg.pickle(Vocab, pickle_vocab, unpickle_vocab)



# coding: utf8
from __future__ import unicode_literals

from .stop_words import STOP_WORDS
from .lex_attrs import LEX_ATTRS

from ..norm_exceptions import BASE_NORMS
from ...language import Language
from ...attrs import LANG


class HindiDefaults(Language.Defaults):
    lex_attr_getters = dict(Language.Defaults.lex_attr_getters)
    lex_attr_getters.update(LEX_ATTRS)
    lex_attr_getters[LANG] = lambda text: 'hi'
    stop_words = STOP_WORDS


class Hindi(Language):
    lang = 'hi'
    Defaults = HindiDefaults


__all__ = ['Hindi']



# coding: utf8
from __future__ import unicode_literals


"""
Example sentences to test spaCy and its language models.

>>> from spacy.lang.zh.examples import sentences
>>> docs = nlp.pipe(sentences)
"""


sentences = [
    "蘋果公司正考量用一億元買下英國的新創公司",
    "自駕車將保險責任歸屬轉移至製造商",
    "舊金山考慮禁止送貨機器人在人行道上行駛",
    "倫敦是英國的大城市"
]



# encoding: utf8
from __future__ import unicode_literals


STOP_WORDS = set("""
alle allerede alt and andre annen annet at av

bak bare bedre beste blant ble bli blir blitt bris by både

da dag de del dem den denne der dermed det dette disse drept du

eller en enn er et ett etter

fem fikk fire fjor flere folk for fortsatt fotball fra fram frankrike fredag
funnet få får fått før først første

gang gi gikk gjennom gjorde gjort gjør gjøre god godt grunn gå går

ha hadde ham han hans har hele helt henne hennes her hun hva hvor hvordan
hvorfor

i ifølge igjen ikke ingen inn

ja jeg

kamp kampen kan kl klart kom komme kommer kontakt kort kroner kunne kveld
kvinner

la laget land landet langt leder ligger like litt løpet lørdag

man mandag mange mannen mars med meg mellom men mener menn mennesker mens mer
millioner minutter mot msci mye må mål måtte

ned neste noe noen nok norge norsk norske ntb ny nye nå når

og også om onsdag opp opplyser oslo oss over

personer plass poeng politidistrikt politiet president prosent på

regjeringen runde rundt russland

sa saken samme sammen samtidig satt se seg seks selv senere september ser sett
siden sier sin sine siste sitt skal skriver skulle slik som sted stedet stor
store står sverige svært så søndag

ta tatt tid tidligere til tilbake tillegg tirsdag to tok torsdag tre tror
tyskland

under usa ut uten utenfor

vant var ved veldig vi videre viktig vil ville viser vår være vært

å år

ønsker
""".split())



# coding: utf8
from __future__ import unicode_literals

from .templates import TPL_DEP_SVG, TPL_DEP_WORDS, TPL_DEP_ARCS
from .templates import TPL_ENT, TPL_ENTS, TPL_FIGURE, TPL_TITLE, TPL_PAGE
from ..util import minify_html


class DependencyRenderer(object):
    """Render dependency parses as SVGs."""
    style = 'dep'

    def __init__(self, options={}):
        """Initialise dependency renderer.

        options (dict): Visualiser-specific options (compact, word_spacing,
            arrow_spacing, arrow_width, arrow_stroke, distance, offset_x,
            color, bg, font)
        """
        self.compact = options.get('compact', False)
        self.word_spacing = options.get('word_spacing', 45)
        self.arrow_spacing = options.get('arrow_spacing',
                                         12 if self.compact else 20)
        self.arrow_width = options.get('arrow_width',
                                       6 if self.compact else 10)
        self.arrow_stroke = options.get('arrow_stroke', 2)
        self.distance = options.get('distance', 150 if self.compact else 175)
        self.offset_x = options.get('offset_x', 50)
        self.color = options.get('color', '#000000')
        self.bg = options.get('bg', '#ffffff')
        self.font = options.get('font', 'Arial')

    def render(self, parsed, page=False, minify=False):
        """Render complete markup.

        parsed (list): Dependency parses to render.
        page (bool): Render parses wrapped as full HTML page.
        minify (bool): Minify HTML markup.
        RETURNS (unicode): Rendered SVG or HTML markup.
        """
        rendered = [self.render_svg(i, p['words'], p['arcs'])
                    for i, p in enumerate(parsed)]
        if page:
            content = ''.join([TPL_FIGURE.format(content=svg)
                               for svg in rendered])
            markup = TPL_PAGE.format(content=content)
        else:
            markup = ''.join(rendered)
        if minify:
            return minify_html(markup)
        return markup

    def render_svg(self, render_id, words, arcs):
        """Render SVG.

        render_id (int): Unique ID, typically index of document.
        words (list): Individual words and their tags.
        arcs (list): Individual arcs and their start, end, direction and label.
        RETURNS (unicode): Rendered SVG markup.
        """
        self.levels = self.get_levels(arcs)
        self.highest_level = len(self.levels)
        self.offset_y = self.distance/2*self.highest_level+self.arrow_stroke
        self.width = self.offset_x+len(words)*self.distance
        self.height = self.offset_y+3*self.word_spacing
        self.id = render_id
        words = [self.render_word(w['text'], w['tag'], i)
                 for i, w in enumerate(words)]
        arcs = [self.render_arrow(a['label'], a['start'],
                                  a['end'], a['dir'], i)
                for i, a in enumerate(arcs)]
        content = ''.join(words) + ''.join(arcs)
        return TPL_DEP_SVG.format(id=self.id, width=self.width,
                                  height=self.height, color=self.color,
                                  bg=self.bg, font=self.font, content=content)

    def render_word(self, text, tag, i):
        """Render individual word.

        text (unicode): Word text.
        tag (unicode): Part-of-speech tag.
        i (int): Unique ID, typically word index.
        RETURNS (unicode): Rendered SVG markup.
        """
        y = self.offset_y+self.word_spacing
        x = self.offset_x+i*self.distance
        return TPL_DEP_WORDS.format(text=text, tag=tag, x=x, y=y)

    def render_arrow(self, label, start, end, direction, i):
        """Render indivicual arrow.

        label (unicode): Dependency label.
        start (int): Index of start word.
        end (int): Index of end word.
        direction (unicode): Arrow direction, 'left' or 'right'.
        i (int): Unique ID, typically arrow index.
        RETURNS (unicode): Rendered SVG markup.
        """
        level = self.levels.index(end-start)+1
        x_start = self.offset_x+start*self.distance+self.arrow_spacing
        y = self.offset_y
        x_end = (self.offset_x+(end-start)*self.distance+start*self.distance
                 - self.arrow_spacing*(self.highest_level-level)/4)
        y_curve = self.offset_y-level*self.distance/2
        if self.compact:
            y_curve = self.offset_y-level*self.distance/6
        if y_curve == 0 and len(self.levels) > 5:
            y_curve = -self.distance
        arrowhead = self.get_arrowhead(direction, x_start, y, x_end)
        arc = self.get_arc(x_start, y, y_curve, x_end)
        return TPL_DEP_ARCS.format(id=self.id, i=i, stroke=self.arrow_stroke,
                                   head=arrowhead, label=label, arc=arc)

    def get_arc(self, x_start, y, y_curve, x_end):
        """Render individual arc.

        x_start (int): X-coordinate of arrow start point.
        y (int): Y-coordinate of arrow start and end point.
        y_curve (int): Y-corrdinate of Cubic Bézier y_curve point.
        x_end (int): X-coordinate of arrow end point.
        RETURNS (unicode): Definition of the arc path ('d' attribute).
        """
        template = "M{x},{y} C{x},{c} {e},{c} {e},{y}"
        if self.compact:
            template = "M{x},{y} {x},{c} {e},{c} {e},{y}"
        return template.format(x=x_start, y=y, c=y_curve, e=x_end)

    def get_arrowhead(self, direction, x, y, end):
        """Render individual arrow head.

        direction (unicode): Arrow direction, 'left' or 'right'.
        x (int): X-coordinate of arrow start point.
        y (int): Y-coordinate of arrow start and end point.
        end (int): X-coordinate of arrow end point.
        RETURNS (unicode): Definition of the arrow head path ('d' attribute).
        """
        if direction is 'left':
            pos1, pos2, pos3 = (x, x-self.arrow_width+2, x+self.arrow_width-2)
        else:
            pos1, pos2, pos3 = (end, end+self.arrow_width-2,
                                end-self.arrow_width+2)
        arrowhead = (pos1, y+2, pos2, y-self.arrow_width, pos3,
                     y-self.arrow_width)
        return "M{},{} L{},{} {},{}".format(*arrowhead)

    def get_levels(self, arcs):
        """Calculate available arc height "levels".
        Used to calculate arrow heights dynamically and without wasting space.

        args (list): Individual arcs and their start, end, direction and label.
        RETURNS (list): Arc levels sorted from lowest to highest.
        """
        levels = set(map(lambda arc: arc['end'] - arc['start'], arcs))
        return sorted(list(levels))


class EntityRenderer(object):
    """Render named entities as HTML."""
    style = 'ent'

    def __init__(self, options={}):
        """Initialise dependency renderer.

        options (dict): Visualiser-specific options (colors, ents)
        """
        colors = {'ORG': '#7aecec', 'PRODUCT': '#bfeeb7', 'GPE': '#feca74',
                  'LOC': '#ff9561', 'PERSON': '#aa9cfc', 'NORP': '#c887fb',
                  'FACILITY': '#9cc9cc', 'EVENT': '#ffeb80', 'LAW': '#ff8197',
                  'LANGUAGE': '#ff8197', 'WORK_OF_ART': '#f0d0ff',
                  'DATE': '#bfe1d9', 'TIME': '#bfe1d9', 'MONEY': '#e4e7d2',
                  'QUANTITY': '#e4e7d2', 'ORDINAL': '#e4e7d2',
                  'CARDINAL': '#e4e7d2', 'PERCENT': '#e4e7d2'}
        colors.update(options.get('colors', {}))
        self.default_color = '#ddd'
        self.colors = colors
        self.ents = options.get('ents', None)

    def render(self, parsed, page=False, minify=False):
        """Render complete markup.

        parsed (list): Dependency parses to render.
        page (bool): Render parses wrapped as full HTML page.
        minify (bool): Minify HTML markup.
        RETURNS (unicode): Rendered HTML markup.
        """
        rendered = [self.render_ents(p['text'], p['ents'],
                    p.get('title', None)) for p in parsed]
        if page:
            docs = ''.join([TPL_FIGURE.format(content=doc)
                            for doc in rendered])
            markup = TPL_PAGE.format(content=docs)
        else:
            markup = ''.join(rendered)
        if minify:
            return minify_html(markup)
        return markup

    def render_ents(self, text, spans, title):
        """Render entities in text.

        text (unicode): Original text.
        spans (list): Individual entity spans and their start, end and label.
        title (unicode or None): Document title set in Doc.user_data['title'].
        """
        markup = ''
        offset = 0
        for span in spans:
            label = span['label']
            start = span['start']
            end = span['end']
            entity = text[start:end]
            fragments = text[offset:start].split('\n')
            for i, fragment in enumerate(fragments):
                markup += fragment
                if len(fragments) > 1 and i != len(fragments)-1:
                    markup += '</br>'
            if self.ents is None or label.upper() in self.ents:
                color = self.colors.get(label.upper(), self.default_color)
                markup += TPL_ENT.format(label=label, text=entity, bg=color)
            else:
                markup += entity
            offset = end
        markup += text[offset:]
        markup = TPL_ENTS.format(content=markup, colors=self.colors)
        if title:
            markup = TPL_TITLE.format(title=title) + markup
        return markup



# coding: utf-8
from __future__ import unicode_literals

from io import StringIO, BytesIO
from pathlib import Path
import pytest

from .util import load_test_model
from ..tokens import Doc
from ..strings import StringStore
from .. import util


# These languages are used for generic tokenizer tests – only add a language
# here if it's using spaCy's tokenizer (not a different library)
# TODO: re-implement generic tokenizer tests
_languages = ['bn', 'da', 'de', 'en', 'es', 'fi', 'fr', 'ga', 'he', 'hu', 'id',
              'it', 'nb', 'nl', 'pl', 'pt', 'ru', 'sv', 'tr', 'xx']
_models = {'en': ['en_core_web_sm'],
           'de': ['de_core_news_md'],
           'fr': ['fr_core_news_sm'],
           'xx': ['xx_ent_web_md'],
           'en_core_web_md': ['en_core_web_md'],
           'es_core_news_md': ['es_core_news_md']}


# only used for tests that require loading the models
# in all other cases, use specific instances

@pytest.fixture(params=_models['en'])
def EN(request):
    return load_test_model(request.param)


@pytest.fixture(params=_models['de'])
def DE(request):
    return load_test_model(request.param)


@pytest.fixture(params=_models['fr'])
def FR(request):
    return load_test_model(request.param)


@pytest.fixture()
def RU(request):
    pymorphy = pytest.importorskip('pymorphy2')
    return util.get_lang_class('ru')()


#@pytest.fixture(params=_languages)
#def tokenizer(request):
    #lang = util.get_lang_class(request.param)
    #return lang.Defaults.create_tokenizer()


@pytest.fixture
def tokenizer():
    return util.get_lang_class('xx').Defaults.create_tokenizer()


@pytest.fixture
def en_tokenizer():
    return util.get_lang_class('en').Defaults.create_tokenizer()


@pytest.fixture
def en_vocab():
    return util.get_lang_class('en').Defaults.create_vocab()


@pytest.fixture
def en_parser(en_vocab):
    nlp = util.get_lang_class('en')(en_vocab)
    return nlp.create_pipe('parser')


@pytest.fixture
def es_tokenizer():
    return util.get_lang_class('es').Defaults.create_tokenizer()


@pytest.fixture
def de_tokenizer():
    return util.get_lang_class('de').Defaults.create_tokenizer()


@pytest.fixture
def fr_tokenizer():
    return util.get_lang_class('fr').Defaults.create_tokenizer()


@pytest.fixture
def hu_tokenizer():
    return util.get_lang_class('hu').Defaults.create_tokenizer()


@pytest.fixture
def fi_tokenizer():
    return util.get_lang_class('fi').Defaults.create_tokenizer()


@pytest.fixture
def id_tokenizer():
    return util.get_lang_class('id').Defaults.create_tokenizer()


@pytest.fixture
def sv_tokenizer():
    return util.get_lang_class('sv').Defaults.create_tokenizer()


@pytest.fixture
def bn_tokenizer():
    return util.get_lang_class('bn').Defaults.create_tokenizer()


@pytest.fixture
def ga_tokenizer():
    return util.get_lang_class('ga').Defaults.create_tokenizer()


@pytest.fixture
def he_tokenizer():
    return util.get_lang_class('he').Defaults.create_tokenizer()


@pytest.fixture
def nb_tokenizer():
    return util.get_lang_class('nb').Defaults.create_tokenizer()

@pytest.fixture
def da_tokenizer():
    return util.get_lang_class('da').Defaults.create_tokenizer()

@pytest.fixture
def ja_tokenizer():
    janome = pytest.importorskip("janome")
    return util.get_lang_class('ja').Defaults.create_tokenizer()


@pytest.fixture
def th_tokenizer():
    pythainlp = pytest.importorskip("pythainlp")
    return util.get_lang_class('th').Defaults.create_tokenizer()

@pytest.fixture
def tr_tokenizer():
    return util.get_lang_class('tr').Defaults.create_tokenizer()


@pytest.fixture
def ru_tokenizer():
    pymorphy = pytest.importorskip('pymorphy2')
    return util.get_lang_class('ru').Defaults.create_tokenizer()


@pytest.fixture
def stringstore():
    return StringStore()


@pytest.fixture
def en_entityrecognizer():
     return util.get_lang_class('en').Defaults.create_entity()


@pytest.fixture
def text_file():
    return StringIO()


@pytest.fixture
def text_file_b():
    return BytesIO()


def pytest_addoption(parser):
    parser.addoption("--models", action="store_true",
        help="include tests that require full models")
    parser.addoption("--vectors", action="store_true",
        help="include word vectors tests")
    parser.addoption("--slow", action="store_true",
        help="include slow tests")

    for lang in _languages + ['all']:
        parser.addoption("--%s" % lang, action="store_true", help="Use %s models" % lang)
    for model in _models:
        if model not in _languages:
            parser.addoption("--%s" % model, action="store_true", help="Use %s model" % model)


def pytest_runtest_setup(item):
    for opt in ['models', 'vectors', 'slow']:
        if opt in item.keywords and not item.config.getoption("--%s" % opt):
            pytest.skip("need --%s option to run" % opt)

    # Check if test is marked with models and has arguments set, i.e. specific
    # language. If so, skip test if flag not set.
    if item.get_marker('models'):
        for arg in item.get_marker('models').args:
            if not item.config.getoption("--%s" % arg) and not item.config.getoption("--all"):
                pytest.skip("need --%s or --all option to run" % arg)



# coding: utf8
from __future__ import unicode_literals


VERBS = set("""
aah abacinate abandon abase abash abate abbreviate abdicate abduce abduct
aberrate abet abhor abide abjure ablactate ablate abnegate abolish abominate
abort abound about-face abrade abrase abreact abridge abrogate abscise abscond
abseil absent absolve absorb absquatulate abstain abstract abuse abut aby abye
accede accelerate accent accentuate accept access accession acclaim acclimate
acclimatise acclimatize accommodate accompany accomplish accord accost account
accouter accoutre accredit accrete accrue acculturate accumulate accurse accuse
accustom ace acerbate acetify acetylate acetylise acetylize ache achieve
achromatise achromatize acid-wash acidify acidulate acknowledge acquaint
acquiesce acquire acquit act action activate actualise actualize actuate
acuminate ad-lib adapt add addict addle address adduce adduct adhere adjoin
adjourn adjudge adjudicate adjure adjust admeasure administer administrate
admire admit admix admonish adolesce adopt adore adorn adsorb adulate adulterate
adumbrate advance advantage advect adventure advert advertise advertize advise
advocate aerate aerify aerosolise aerosolize aestivate affect affiance affiliate
affirm affix afflict afford afforest affranchise affright affront age agenise
agenize agglomerate agglutinate aggrade aggrandise aggrandize aggravate
aggregate aggress aggrieve aggroup agitate agnise agnize agonise agonize agree
aid ail aim air air-condition air-cool air-drop air-freight air-ship air-slake
airbrush airfreight airlift airmail airt alarm alchemise alchemize alcoholise
alcoholize alert alibi alien alienate alight align aliment aline alkalify
alkalinise alkalinize alkalise alkalize allay allege allegorise allegorize
alleviate alligator alliterate allocate allot allow allowance alloy allude
allure ally alphabetise alphabetize alter altercate alternate aluminise
aluminize amalgamate amass amaze ambition amble ambulate ambuscade ambush
ameliorate amend amerce americanise americanize ammoniate ammonify amnesty
amortise amortize amount amplify amputate amuse anaesthetise anaesthetize
anagram anagrammatise anagrammatize analogise analogize analyse analyze
anastomose anathematise anathematize anathemise anathemize anatomise anatomize
anchor ancylose anele anesthetise anesthetize anger angle angle-park anglicise
anglicize anguish angulate animadvert animalise animalize animate animise
animize ankylose anneal annex annihilate annotate announce annoy annul
annunciate anodise anodize anoint answer antagonise antagonize ante antecede
antedate anthologise anthologize anthropomorphise anthropomorphize antic
anticipate antiquate antique antisepticize ape aphorise aphorize apologise
apologize apostatise apostatize apostrophise apostrophize apotheose apotheosise
apotheosize appal appall apparel appeal appear appease append apperceive
appertain applaud applique apply appoint apportion appose appraise appreciate
apprehend apprentice apprise apprize approach approbate appropriate approve
approximate aquaplane aquatint arbitrage arbitrate arborise arborize arc arch
archaise archaize archive argue argufy arise arm armor armour aromatise
aromatize arouse arraign arrange array arrest arrive arrogate arterialise
arterialize article articulate ascend ascertain ascribe ash ask asperse asphalt
asphyxiate aspirate aspire assail assassinate assault assay assemble assent
assert assess asseverate assibilate assign assimilate assist associate assoil
assonate assort assuage assume assure asterisk astonish astound astringe
astrogate atom-bomb atomise atomize atone atrophy attach attack attain attaint
attemper attempt attend attenuate attest attire attitudinise attitudinize attorn
attract attribute attune auction auctioneer audit audition augment augur aurify
auscultate auspicate authenticate author authorise authorize autoclave autograph
automate automatise automatize automobile autopsy autotomise autotomize avail
avalanche avenge aver average avert avianise avianize aviate avoid avouch avow
avulse await awake awaken award awe ax axe azure baa babbitt babble baby baby-
sit bach bachelor back backbite backcross backdate backfire background backhand
backlash backlog backpack backpedal backscatter backslap backslide backspace
backstitch backstop backstroke backtrack bacterise bacterize badge badger
badmouth baffle bag bail bait bake balance bald bale balk balkanise balkanize
ball ballast balloon ballot ballyhoo ballyrag bamboozle ban band bandage bandy
bang banish bank bankroll bankrupt banquet banter baptise baptize bar barb
barbarise barbarize barbecue barbeque barber bard barde bare barf bargain barge
bark barnstorm baronetise baronetize barrack barrage barrel barricade barricado
barter base bash basify bask basset bastardise bastardize baste bastinado bat
batch bate batfowl bath bathe batik batten batter battle baulk bawl bay bayonet
be beach beacon bead beak beam bean bear beard beat beatify beautify beaver
bebop becalm bechance becharm beckon becloud become bed bed-hop bedamn bedaub
bedaze bedazzle bedeck bedevil bedew bedhop bedight bedim bedizen bedraggle beef
beep beeswax beetle befall befit befog befool befoul befriend befuddle beg beget
beggar begild begin begrime begrudge beguile behave behead behold behoove behove
bejewel belabor belabour belay belch beleaguer belie believe belittle bell
bellow belly belly-flop belly-land bellyache bellylaugh belong belt bemire
bemoan bemock bemuse bench bend benday benefact benefice beneficiate benefit
benficiate benight benumb beplaster bequeath berate bereave berry berth beseech
beseem beset beshrew besiege besmear besmirch besot bespangle bespatter bespeak
bespeckle bespot besprinkle best bestialise bestialize bestir bestow bestrew
bestride bet bethink betide betoken betray betroth better bevel bewail beware
bewhisker bewilder bewitch bewray bias bib bicker bicycle bid bide biff
bifurcate bight bike bilge bilk bill billet billow bin bind binge bioassay
biodegrade birch bird bird-nest birdie birdlime birdnest birdwatch birl birle
birr birth bisect bitch bite bitt bitter bituminise bituminize bivouac blab
blabber black blackball blackberry blacken blackguard blackjack blacklead
blackleg blacklist blackmail blacktop blackwash blame blanch blandish blank
blanket blare blarney blaspheme blast blat blate blather blaze blazon bleach
blear bleat bleed bleep blemish blench blend bless blether blight blind
blindfold blindside blink blinker blister blither blitz blitzkrieg bloat blob
block blockade blog blood bloody bloom blossom blot blotch bloviate blow blow-
dry blub blubber bludgeon blue blue-pencil blueprint bluff blunder blunt blur
blurt blush bluster board boast boat boat-race bob bobble bobsled bode bodge
body body-surf bodypaint bog bogey boggle boil boldface bollix bollocks
bolshevise bolshevize bolster bolt bomb bombard bombilate bombinate bombproof
bond bonderise bonderize bone bong bonk bonnet boo boob boogie book boom
boomerang boondoggle boost boot bootleg bootlick bootstrap booze bop border bore
borrow bosom boss botanise botanize botch bother bottle bottlefeed bottleneck
bottom bounce bound bourgeon bouse bow bowdlerise bowdlerize bower bowl bowse
box boycott brabble brace brachiate bracket brad brag braid brail braille brain
brainstorm brainwash braise brake branch brand brandish brandmark brattice
brattle brave bravo brawl bray braze brazen breach bread break break-dance
breakfast bream breast breastfeed breaststroke breathalyse breathalyze breathe
brecciate breed breeze brevet brew bribe bridge bridle brief brigade brighten
brim brine bring brisk brisken bristle broach broadcast broaden broadside
brocade broider broil broker bromate brominate bronze brooch brood brook broom
browbeat brown brown-nose browse bruise bruit brunch brush brutalise brutalize
bubble buccaneer buck bucket buckle buckram bud budge budget buff buffalo buffer
buffet bug bugger bugle build bulge bulk bull bulldog bulldoze bulletin
bulletproof bullshit bully bullyrag bulwark bum bumble bump bunch bunco bundle
bung bungle bunk bunker bunt buoy bur burble burden burgeon burglarise
burglarize burgle burke burl burlesque burn burnish burp burr burrow burst
burthen bury bus bush bushel bushwhack busk buss bust bustle busy butcher butt
butt-weld butter butterfly button buttonhole buttress buttweld butylate buy buzz
by-bid bypass ca-ca cab cabal cabbage cabin cable cache cachinnate cackel cackle
caddie caddy cadge cage cajole cake cakewalk calcify calcimine calcine calculate
calendar calender calibrate caliper calk call calligraph calliper callous callus
calm calumniate calve camber camouflage camp campaign camphorate can canal
canalise canalize cancel candle candy cane canker cannibalise cannibalize cannon
cannonade cannulate cannulise cannulize canoe canonise canonize canoodle canopy
cant canter cantilever cantillate canton canulate canvas canvass cap capacitate
caparison caper capitalise capitalize capitulate caponise caponize capriole
capsize capsulate capsule capsulise capsulize captain caption captivate capture
caracole caramelise caramelize caravan carbonate carbonise carbonize carboxylate
carburet carburise carburize card care careen career caress caricature carjack
cark carmine carnalise carnalize carnify carol carom carouse carp carpenter
carpet carry cart cartoon cartwheel carve cascade case caseate cash cashier
casket cast castigate castle castrate cat catabolise catabolize catalog
catalogue catalyse catalyze catapult catcall catch catechise catechize
categorise categorize catenate catenulate cater caterwaul cathect catheterise
catheterize catholicise catholicize catnap caucus caulk cause causeway cauterise
cauterize caution cave cavern cavil cavort caw cease cede celebrate cement cense
censor censure census center centralise centralize centre centrifugate
centrifuge cere cerebrate certificate certify cha-cha chafe chaff chaffer
chagrin chain chain-smoke chair chairman chalk challenge chamber chamfer champ
champion chance chandelle change channel channel-surf channelise channelize
chant chap chaperon chaperone char character characterise characterize charcoal
charge chariot charleston charm chart charter chase chasse chasten chastise chat
chatter chauffeur chaw cheapen cheat check checker checkmate checkrow cheek
cheep cheer cheerlead cheese chelate chemisorb cheque chequer cherish chevvy
chevy chew chicane chicken-fight chickenfight chide child-proof childproof chill
chime chin chine chink chip chirk chiromance chirp chirr chirrup chisel chit-
chat chitchat chitter chivvy chivy chlorinate chloroform chock choir choke chomp
chondrify choose chop chord choreograph chortle chorus chouse christen
christianise christianize chrome chromium-plate chronicle chronologise
chronologize chuck chuckle chuff chug chunk church churn churr chute cicatrise
cicatrize cinch cinematise cinematize cipher circle circuit circularise
circularize circulate circumambulate circumcise circumfuse circumnavigate
circumscribe circumstantiate circumvallate circumvent circumvolute circumvolve
cite citify citrate civilise civilize clabber clack claim clam clamber clamor
clamour clamp clang clangor clangour clank clap clapboard clapperclaw claret
clarify clarion clash clasp class classicise classicize classify clatter claver
claw claxon clean cleanse clear clear-cut cleat cleave clench clerk clew click
climax climb clinch cling clink clinker clip clitter cloak clobber clock clog
cloister clomp clone clop close closet closure clot clothe cloture cloud clout
clown cloy club cluck clue clump clunk cluster clutch clutter co-author co-
educate co-occur co-opt co-sign co-star co-vary coach coact coagulate coal
coalesce coapt coarsen coast coat coax cobble cobblestone cocainise cocainize
cock cocker cockle coconspire cocoon cod coddle code codify coeducate coerce
coexist coffin cog coggle cogitate cognise cognize cohabit cohere coif coiffe
coiffure coil coin coincide coinsure coke cold-cream cold-shoulder coldcock
coldwork collaborate collapse collar collate collateralize collect collectivise
collectivize collide colligate collimate collocate collogue collude colly
colonise colonize color colorcast colorise colorize colour colourise colourize
comb combat combine combust come comfit comfort command commandeer commemorate
commence commend comment commentate commercialise commercialize comminate
commingle comminute commiserate commission commit commix commove communalise
communalize commune communicate communise communize commutate commute compact
companion company compare comparison-shop compart compartmentalise
compartmentalize compass compassionate compel compensate compere compete compile
complain complect complement complete complexify complexion complicate
compliment complot comply comport compose compost compound comprehend compress
comprise compromise compute computerise computerize con concatenate conceal
concede conceive concenter concentrate concentre conceptualise conceptualize
concern concert concertina concertise concertize conciliate conclude concoct
concord concrete concretise concretize concur concuss condemn condense
condescend condition condole condone conduce conduct cone confab confabulate
confect confection confederate confer confess confide configure confine confirm
confiscate conflagrate conflate conflict conform confound confront confuse
confute conga conge congeal congee congest conglobate conglobe conglomerate
conglutinate congratulate congregate conjecture conjoin conjugate conjure conk
conn connect connive connote conquer conscript consecrate consent conserve
consider consign consist consociate console consolidate consonate consort
conspire constellate consternate constipate constitute constitutionalise
constitutionalize constrain constrict constringe construct construe
consubstantiate consult consume consummate contact contain containerise
containerize contaminate contemn contemplate contemporise contemporize contend
content contest continue contort contour contract contradance contradict
contradistinguish contraindicate contrast contravene contredanse contribute
contrive control controvert contuse convalesce convect convene conventionalise
conventionalize converge converse convert convey convict convince convoke
convolute convolve convoy convulse coo cooccur cook cool cooper cooperate
coordinate cop cope copolymerise copolymerize copper copper-bottom copulate copy
copyedit copyread copyright coquet coquette corbel cord corduroy core cork
corkscrew corn corner cornice coronate corrade corral correct correlate
correspond corroborate corrode corrugate corrupt corset coruscate cosh cosign
cosponsor cosset cost costume cotton couch cough counsel count countenance
counter counter-drill counteract counterattack counterbalance counterchallenge
counterchange countercheck counterclaim counterfeit countermand countermarch
countermine counterplot counterpoint counterpoise counterpose countersign
countersink counterstrike countervail counterweight country-dance couple course
court court-martial covenant cover covet cow cower cowhide cowl cox cozen crab
crack crackle cradle craft cram cramp cranch crane crank crap crape crash crash-
dive crate craunch crave crawfish crawl crayon craze creak cream crease create
credit creep cremate crenel crenelate crenellate creolize creosote crepe
crepitate crescendo crest crew crib crick cricket criminalise criminalize
criminate crimp crimson cringe crinkle cripple crisp crispen crisscross
criticise criticize critique croak crochet crock cronk crook croon crop croquet
cross cross-check cross-dress cross-fertilise cross-fertilize cross-file cross-
index cross-link cross-pollinate cross-refer crossbreed crosscut crosshatch
crossruff crouch crow crowd crown crucify cruise crumb crumble crump crumple
crunch crusade crush crust cry crystalise crystalize crystallise crystallize cub
cube cuckold cuckoo cuddle cudgel cue cuff cull culminate cultivate culture
cumber cumulate cup curb curdle cure curl curry currycomb curse curtail curtain
curtsey curtsy curve curvet cushion cuss custom-make customise customize cut
cutinize cybernate cycle cyclostyle cypher dab dabble dado daisy-chain dally dam
damage damascene damn damp dampen dance dandify dandle dangle dapple dare darken
darn dart dash date date-mark dateline datemark daub daunt dawdle dawn daydream
daze dazzle de-access de-aerate de-emphasise de-emphasize de-energise de-
energize de-escalate de-ice de-iodinate de-ionate deaccession deactivate deaden
deaerate deaf deafen deal deaminate deaminize debar debark debase debate debauch
debilitate debit debone debouch debrief debug debunk debut decaffeinate
decalcify decamp decant decapitate decarbonate decarbonise decarbonize
decarboxylate decarburise decarburize decay decease deceive decelerate
decentralise decentralize decerebrate decertify dechlorinate decide decimalise
decimalize decimate decipher deck declaim declare declassify declaw decline
declutch decoct decode decoke decollate decolonise decolonize decolor decolorise
decolorize decolour decolourise decolourize decommission decompose decompress
deconcentrate deconsecrate deconstruct decontaminate decontrol decorate
decorticate decouple decoy decrease decree decrepitate decrescendo decriminalise
decriminalize decry decrypt decussate dedicate dedifferentiate deduce deduct
deem deep-dye deep-fat-fry deep-fry deep-six deepen deface defalcate defame
defang defat default defeat defecate defect defeminise defeminize defend
defenestrate defer defervesce defibrillate defibrinate defile define deflagrate
deflate deflect deflower defog defoliate deforest deform defraud defray defrock
defrost defuse defy degas degauss degenerate deglaze deglycerolise deglycerolize
degrade degrease degust dehisce dehorn dehumanise dehumanize dehumidify
dehydrate dehydrogenate deice deify deign deionize deject delay delegate delete
deliberate delight delimit delimitate delineate deliquesce delist deliver
delocalize delouse delude deluge delve demagnetise demagnetize demand demarcate
demasculinise demasculinize dematerialise dematerialize demean demilitarise
demilitarize demineralise demineralize demise demist demo demob demobilise
demobilize democratise democratize demodulate demolish demonetise demonetize
demonise demonize demonstrate demoralise demoralize demote demulsify demur
demyelinate demystify demythologise demythologize denationalise denationalize
denaturalise denaturalize denature denazify denigrate denitrify denominate
denote denounce dent denudate denude deny deodorise deodorize deodourise
deoxidise deoxidize deoxygenate depart depend depersonalise depersonalize depict
depilate deplane deplete deplore deploy deplumate deplume depolarise depolarize
depone depopulate deport depose deposit deprave deprecate depreciate depress
depressurise depressurize deprive depute deputise deputize deracinate derail
derange derate derecognise derecognize deregulate derequisition derestrict
deride derive derogate desacralize desalinate desalinise desalinize desalt
descale descant descend describe descry desecrate desegregate desensitise
desensitize desert deserve desex desexualise desexualize desiccate design
designate desire desist desolate desorb despair despatch despise despoil despond
desquamate destabilise destabilize destain destalinise destalinize destine
destress destroy destruct desulfurize desulphurize desynchronise desynchronize
detach detail detain detect deter deterge deteriorate determine detest dethaw
dethrone detick detonate detour detox detoxicate detoxify detract detrain
detransitivise detransitivize detribalise detribalize detusk devaluate devalue
devastate devein develop deviate devil devilise devilize devise devitalise
devitalize devitrify devoice devolve devote devour diabolise diabolize diagnose
diagonalise diagonalize diagram dial dialyse dialyze diazotize dibble dice
dichotomise dichotomize dicker dictate didder diddle die diet differ
differentiate diffract diffuse dig digest digitalise digitalize digitise
digitize dignify digress dike dilapidate dilate dilly-dally dillydally dilute
dim dimension diminish dimple din dine ding dingdong dinge dip diphthongise
diphthongize direct dirty disable disabuse disaccord disadvantage disaffect
disafforest disagree disallow disambiguate disappear disappoint disapprove
disarm disarrange disarray disarticulate disassemble disassociate disavow
disband disbar disbelieve disbud disburden disburse disc-jockey discard discase
discern discerp discharge discipline disclaim disclose disco discolor
discolorise discolorize discolour discolourise discombobulate discomfit
discommode discompose disconcert disconnect discontent discontinue discord
discount discountenance discourage discourse discover discredit discriminate
discuss disdain disembark disembarrass disembody disembowel disembroil disenable
disenchant disencumber disenfranchise disengage disentangle disestablish
disesteem disfavor disfavour disfigure disforest disfranchise disgorge disgrace
disgruntle disguise disgust dish disharmonize dishearten dishevel dishonor
dishonour disillusion disincarnate disincline disinfect disinfest disinherit
disintegrate disinter disinvest disinvolve disjoin disjoint disk disk-jockey
dislike dislocate dislodge dismantle dismay dismember dismiss dismount disobey
disoblige disorder disorganise disorganize disorient disorientate disown
disparage dispatch dispel dispense disperse dispirit displace display displease
displume disport dispose dispossess dispread disprove dispute disqualify
disquiet disregard disrespect disrobe disrupt diss dissatisfy dissect dissemble
disseminate dissent dissertate dissever dissimilate dissimulate dissipate
dissociate dissolve dissonate dissuade distance distemper distend distil distill
distinguish distort distract distrain distress distribute district distrust
disturb disunify disunite ditch dither ditto divagate divaricate dive dive-bomb
diverge diversify divert divest divide divine divorce divulge dizen dizzy dj do
dock docket doctor document dodder dodge doff dog dogfight dogmatise dogmatize
dogsled domesticate domesticise domesticize domicile domiciliate dominate
domineer don donate dong doodle doom dope dose doss dot dote double double-check
double-date double-glaze double-park double-space double-team doubt douche douse
dovetail dower down downgrade download downplay downsize dowse doze draft drag
draggle dragoon drain dramatise dramatize drape draught draw drawl dread dream
dredge drench dress dribble drift drill drink drip drip-dry drive drivel drizzle
drone drool droop drop drop-kick dropforge dropkick drown drowse drub drudge
drug drum dry dry-dock dry-nurse dry-rot dry-wall drydock dub duck duel dulcify
dulcorate dull dumbfound dummy dump dun dung dunk dupe duplex duplicate dusk
dust dwarf dwell dwindle dye dyke dynamise dynamite dynamize e-mail eagle
earmark earn earth ease eat eavesdrop ebb ebonise ebonize echo eclipse economise
economize eddy edge edify edit editorialise editorialize educate educe
edulcorate eff efface effect effectuate effeminise effeminize effervesce
effloresce effuse egest egg egotrip egress ejaculate eject elaborate elapse
elate elbow elect electioneer electrify electrocute electroplate elegise elegize
elevate elicit elide eliminate elocute elongate elope elucidate elude elute
emaciate email emanate emancipate emasculate embalm embank embargo embark
embarrass embattle embed embellish embezzle embitter emblazon embody embolden
emboss embower embrace embrangle embrittle embrocate embroider embroil embrown
emcee emend emerge emigrate emit emote empale empanel empathise empathize
emphasise emphasize emplace emplane employ empower empty empurple emulate
emulsify enable enact enamel enamor enamour encamp encapsulate encase enchain
enchant encipher encircle enclose enclothe encode encompass encore encounter
encourage encrimson encroach encrust encrypt encumber end endanger endear
endeavor endeavour endorse endow endue endure energise energize enervate
enfeeble enfeoff enfilade enfold enforce enfranchise engage engender engild
engineer englut engorge engraft engrave engross engulf enhance enjoin enjoy
enkindle enlace enlarge enlighten enlist enliven enmesh ennoble enounce enplane
enquire enrage enrapture enrich enrobe enrol enroll ensconce enshrine enshroud
ensile ensky enslave ensnare ensnarl ensue ensure entail entangle enter
entertain enthral enthrall enthrone enthuse entice entitle entomb entrain
entrance entrap entreat entrench entrust entwine enucleate enumerate enunciate
envelop envenom environ envisage envision envy enwrap epilate epitomise
epitomize epoxy equal equalise equalize equate equilibrate equilibrise
equilibrize equip equivocate eradicate erase erect erode eroticize err eruct
erupt escalade escalate escallop escape eschew escort espouse espy essay
establish esteem esterify estimate estivate estrange etch eternalise eternalize
eternise eternize etherealize etherialise etherify etherise etherize etiolate
etymologise etymologize eulogise eulogize euphemise euphemize europeanise
europeanize evacuate evade evaluate evanesce evangelise evangelize evaporate
even eventuate evert evict evidence evince eviscerate evoke evolve exacerbate
exact exaggerate exalt examine exasperate excavate exceed excel except excerpt
exchange excise excite exclaim exclude excogitate excommunicate excoriate
excrete excruciate exculpate excuse execrate execute exemplify exempt exenterate
exercise exert exfoliate exhale exhaust exhibit exhilarate exhort exhume exile
exist exit exonerate exorcise exorcize expand expatiate expatriate expect
expectorate expedite expel expend expense experience experiment expiate expire
explain explicate explode exploit explore export expose exposit expostulate
expound express express-mail expropriate expunge expurgate exsert exsiccate
extemporise extemporize extend extenuate exteriorise exteriorize exterminate
externalise externalize extinguish extirpate extol extort extract extradite
extrapolate extravasate extricate extrude exuberate exudate exude exult exuviate
eye eyeball eyewitness fabricate face face-harden face-lift facilitate facsimile
factor factorise factorize fade fag faggot fagot fail faint fair fake falcon
fall falsify falter familiarise familiarize famish fan fancify fancy fantasise
fantasize fantasy farce fare farm farrow fart fascinate fashion fast fasten fat
fate father fathom fatigue fatten fault favor favour fawn fax faze fear feast
feather featherbed feature fecundate federalise federalize federate fee fee-tail
feed feel feign feint felicitate fell fellate felt feminise feminize fence fend
ferment ferret ferry fertilise fertilize fester festinate festoon fetch fete
fetishize fetter fettle feud feudalize fib fibrillate fictionalise fictionalize
fiddle fidget field field-test fight figure filch file filet filiate filibuster
filigree fill fillet film film-make filter filtrate fin finagle finalise
finalize finance financier find fine fine-tune finedraw finger finger-paint
finger-spell fingerprint fingerspell finish fink fire firebomb fireproof firm
fish fishtail fissure fistfight fit fix fixate fizz fizzle flabbergast flag
flagellate flail flake flambe flame flank flap flare flash flash-freeze flat-hat
flatten flatter flaunt flavor flavour flaw flay fleck fledge flee fleece fleer
fleet flense flesh flex flick flicker flight flim-flam flinch fling flip flip-
flop flirt flit flitter float flocculate flock flog flood floodlight floor flop
floss flounce flounder flour flourish flout flow flower flub fluctuate fluff
flummox flump flunk fluoresce fluoridate fluoridise fluoridize flurry flush
fluster flute flutter flux fly fly-fish flyfish foal foam fob focalise focalize
focus fodder fog foil foist fold foliate follow foment fondle fool foolproof
foot footle footnote footslog forage foray forbear forbid force force-feed
force-land ford forearm forebode forecast foreclose foredate foredoom forefend
foregather forego foreground foreknow foreordain foresee foreshadow foreshorten
foreshow forest forestall foreswear foretell forewarn forfeit forfend forgather
forge forget forgive forgo fork form formalise formalize format formicate
formularise formularize formulate fornicate forsake forswear fort fortify
forward fossilise fossilize foster foul found founder fowl fox foxhunt foxtrot
fraction fractionate fracture fragment fragmentise fragmentize frame franchise
frank frap fraternise fraternize fray frazzle freak freckle free free-associate
free-base freelance freeload freewheel freeze freeze-dry freight french french-
fry frenchify frequent fresco freshen fress fret fricassee fright frighten
fringe frisk fritter frivol frizz frizzle frock frog frogmarch frolic front
frost froth frown fructify fruit frustrate fry ftp fuck fuddle fudge fuel fulfil
fulfill full fullback fulminate fumble fume fumigate function fund fund-raise
fundraise funk funnel furbish furcate furl furlough furnish furrow further fuse
fusillade fuss fustigate g.i. gab gabble gad gag gage gaggle gain gainsay gall
gallivant gallop galumph galvanise galvanize gamble gambol game gang gang-rape
gangrene gaol gap gape garage garb garble garden gargle garland garment garner
garnish garnishee garotte garrison garrote garrotte garter gas gasconade gash
gasify gasp gate gate-crash gather gauffer gauge gawk gawp gaze gazette gazump
gear gee gel gelatinise gelatinize geld geminate general generalise generalize
generate gentle gentrify genuflect germinate gerrymander gestate gesticulate
gesture get geyser ghettoise ghettoize ghost ghostwrite gi gibber gibbet gibe
gift gift-wrap giggle gild gimp gin ginger gip gird girdle girth give glaciate
gladden glamorise glamorize glamour glamourise glamourize glance glare glass
glaze gleam glean glide glide-bomb glimmer glimpse glint glissade glisten
glitter gloat globalise globalize globe-trot glom glorify glory gloss glow
glower glue glug glut gluttonise gluttonize glycerolise glycerolize gnarl gnash
gnaw go goad gobble goffer goggle gold-plate goldbrick goldplate golf gong goof
goof-proof goofproof google goose gore gorge gormandise gormandize gossip gouge
gourmandize govern gown grab grace gradate grade graduate graft grain grandstand
grant granulate graph grapple grasp grass grate gratify grave gravel gravitate
gray graze grease green greet grey griddle grieve grill grimace grime grin grind
grip gripe grit grizzle groak groan groin grok groom groove grope gross grouch
ground group grouse grout grovel grow growl grub grubstake grudge grumble grunt
grunt-hoot gruntle guarantee guard guess guesstimate guffaw guggle guide
guillotine gull gulp gum gun gurgle gush gut gutter guttle guy guzzle gybe gyp
gyrate habilitate habit habituate hack hackle haemagglutinate haggle hail hale
halloo hallow hallucinate halt halter halve ham hammer hamper hamstring hand
hand-build hand-dye hand-pick hand-wash handbuild handcolor handcolour handcraft
handcuff handicap handle handstamp handwash handwrite hang hanker hap happen
harangue harass harbinger harbor harbour harden hare hark harken harlequin harm
harmonise harmonize harness harp harpoon harrow harry harshen harvest hash hasp
hassle hasten hat hatch hatchel hate haul haunt have haw hawk hay hazard haze
head headbutt headline headquarter heal heap hear hearken hearten heat heave
heckle hectograph hector hedge hedgehop hee-haw heed heel heft heighten heist
heliograph helm help hem hemagglutinate hemorrhage hemstitch hen-peck henna
herald herd hero-worship hesitate heterodyne hew hex hibachi hibernate hiccough
hiccup hide hie higgle high-tail highjack highlight hightail hijack hike hill
hinder hinge hint hire hiss hit hitch hitchhike hive hoard hoax hob hobble
hobnail hobnob hock hoe hog hog-tie hoist hold hole holiday holler hollo hollow
holystone home home-school homer homestead homogenise homogenize homologise
homologize hone honey honeycomb honeymoon honk honor honour hood hoodoo hoodwink
hoof hook hoop hoot hoover hop hop-skip hope hopple horn hornswoggle horrify
horripilate horse horse-race horse-trade horseshoe horsewhip hose hospitalise
hospitalize host hot-dog hot-wire hot-work hotfoot hound house house-train
housebreak houseclean housekeep hover howl huckster huddle hue huff hug hulk
hull hum humanise humanize humble humbug humidify humify humiliate humor humour
hump hunch hunger hunker hunt hurdle hurl hurrah hurry hurt hurtle husband hush
husk hustle hybridise hybridize hydrate hydrogen-bomb hydrogenate hydrolise
hydrolize hydrolyse hydrolyze hydroplane hygienise hygienize hymn hype
hyperbolise hyperbolize hyperextend hypertrophy hyperventilate hyphen hyphenate
hypnotise hypnotize hypophysectomise hypophysectomize hypostatise hypostatize
hypothecate hypothesise hypothesize ice idealise idealize ideate identify idle
idolise idolize ignite ignore ill-treat ill-use illegalise illegalize illume
illuminate illumine illustrate image imagine imbed imbibe imbricate imbrue imbue
imitate immaterialise immaterialize immerse immigrate immingle immix immobilise
immobilize immolate immortalise immortalize immunise immunize immure impact
impair impale impanel impart impeach impede impel impend imperil impersonate
impinge implant implement implicate implode implore imply import importune
impose impound impoverish imprecate impregnate impress imprint imprison improve
improvise improvize impugn impulse-buy impute inactivate inaugurate incandesce
incapacitate incarcerate incarnadine incarnate incase incense inch incinerate
incise incite incline inclose include incommode inconvenience incorporate
increase incriminate incrust incubate inculcate inculpate incur incurvate
indemnify indent indenture index indicate indict indispose indite individualise
individualize individuate indoctrinate indorse induce induct indue indulge
indurate industrialise industrialize indwell inebriate infatuate infect infer
infest infiltrate infix inflame inflate inflect inflict influence inform infract
infringe infuriate infuscate infuse ingeminate ingest ingraft ingrain ingratiate
ingurgitate inhabit inhale inhere inherit inhibit inhume initial initialise
initialize initiate inject injure ink inlay inmarry innervate innovate inoculate
inosculate input inquire inscribe inseminate insert inset insinuate insist
insolate inspan inspect inspire inspirit inspissate instal install instance
instantiate instigate instil instill institute institutionalise institutionalize
instruct instrument instrumentate insufflate insulate insult insure integrate
intend intensify inter interact interbreed intercalate intercede intercept
interchange intercommunicate interconnect interdepend interdict interest
interfere interiorise interiorize interject interlace interlard interleave
interlink interlock interlope interlude intermarry intermediate intermingle
intermit intermix intern internalise internalize internationalise
internationalize interpellate interpenetrate interpolate interpose interpret
interrelate interrogate interrupt intersect intersperse interstratify intertwine
intervene interview interweave intimate intimidate intonate intone intoxicate
intransitivise intransitivize intrench intrigue introduce introject intromit
introspect introvert intrude intrust intubate intuit intumesce intussuscept
inunct inundate inure invade invaginate invalid invalidate inveigh inveigle
invent inventory invert invest investigate invigilate invigorate invite invoice
invoke involve inweave iodinate iodise iodize ionate ionise ionize iridesce irk
iron irradiate irrigate irritate irrupt islamise islamize isolate isomerise
isomerize issue italicise italicize itch itemise itemize iterate itinerate jab
jabber jack jacket jackknife jacklight jackrabbit jactitate jade jag jail jam
jampack jangle japan jar jaundice jaunt jaw jawbone jaywalk jazz jeer jell
jellify jelly jeopardise jeopardize jerk jest jet jettison jewel jib jibe jig
jiggle jilt jimmy jingle jingle-jangle jinx jitterbug jive job jockey jog joggle
join joint joke jollify jolly jolt josh jostle jot jounce journey joust joy
joyride jubilate judder judge jug juggle julienne jumble jump jump-start
jumpstart junk junket junketeer justify jut juxtapose kayak kayo keel keen keep
kennel keratinise keratinize kern key keynote kibbitz kibitz kibosh kick kick-
start kid kidnap kill kindle kink kip kiss kit kite kitten knap knead kneecap
kneel knell knife knight knit knock knot know knuckle kotow kowtow kvetch label
labialise labialize labor labour lace lacerate lack lacquer lactate ladder lade
laden ladle lag laicise laicize lallygag lam lamb lambast lambaste lame lament
laminate lampoon lance land landscape languish lap lapidate lapidify lapse lard
lark larn larrup lash lasso last latch lateralize lather latinise latinize laud
laugh launch launder lave lavish lay layer laze leach lead leaf league leak lean
leap leapfrog learn lease leash leather leave leaven lecture leech leer legalise
legalize legislate legitimate legitimatise legitimatize legitimise legitimize
lend lengthen lenify lessen let letter levant level lever leverage levitate levy
lexicalise lexicalize liaise libel liberalise liberalize liberate librate
licence license lick lie lifehack lift ligate light lighten lighter lignify like
liken lilt limber lime limit limn limp line linearise linearize linger link
lionise lionize lip-read lip-sync lip-synch lipread lipstick liquefy liquidate
liquidise liquidize liquify lisp list listen literalise literalize lithograph
litigate litter live liven load loaf loan loathe lob lobby localise localize
locate lock locomote lodge loft log log-in logroll loiter loll lollop lollygag
long look loom loop loose loosen loot lop lope lord lose lot louden lounge lour
love low lowball lower lube lubricate lucubrate luff lug luge lull lumber
luminesce lump lunch lunge lurch lure lurk lust lustrate luxate luxuriate lynch
lyophilise lyophilize lyric lysogenize macadamise macadamize macerate
machicolate machinate machine machine-wash macrame maculate madden madder
madrigal magnetise magnetize magnify mail maim mainline maintain major make
maledict malfunction malign malinger malnourish malt maltreat malversate mambo
man manacle manage mandate manducate maneuver mangle manhandle manicure manifest
manifold manipulate manoeuver manoeuvre mantle manufacture manumit manure map
mapquest mar maraud marble marbleise marbleize marcel march marginalise
marginalize marinade marinate mark market maroon marry marshal martyr martyrise
martyrize marvel masculinise masculinize mash mask masquerade mass mass-produce
massacre massage master mastermind masticate masturbate mat match mate
materialise materialize matriculate matt-up matte matter maturate mature maul
maunder maximise maximize mean meander measure mechanise mechanize meddle
mediate medicate medicine meditate meet melanise melanize meld meliorate mellow
melodise melodize melt memorialise memorialize memorise memorize menace mend
menstruate mensurate mention mentor meow mercerise mercerize merchandise merge
merit mesh mesmerise mesmerize mess message metabolise metabolize metal metalize
metallize metamorphose metastasise metastasize meter metricate metricise
metricize metrify mew mewl miaou miaow micro-cook microcopy microfilm microwave
micturate middle miff migrate mildew militarise militarize militate milk mill
mime mimeo mimeograph mimic mince mind mine mineralize mingle miniate
miniaturise miniaturize minify minimise minimize minister minstrel mint mire
mirror misaddress misadvise misalign misally misapply misapprehend
misappropriate misbehave misbelieve miscalculate miscall miscarry miscast
miscegenate misconceive misconduct misconstrue miscount miscreate misdate
misdeal misdeliver misdemean misdirect misdo misestimate misfire misfunction
misgauge misgive misgovern misguide mishandle misidentify misinform misinterpret
misjudge mislay mislead mismanage mismarry mismatch mismate misname misperceive
misplace misplay misprint mispronounce misquote misread misremember misrepresent
miss misspeak misspell misspend misstate mist mistake mistime mistranslate
mistreat mistrust misunderstand misuse miter mitigate mix mizzle moan mob
mobilise mobilize mock model moderate modernise modernize modify modulate moil
moisten moisturise moisturize mold molder molest mollify mollycoddle molt
monetise monetize monger mongrelise mongrelize monish monitor monkey monologuise
monologuize monopolise monopolize monumentalise monumentalize moo mooch moon
moonlight moonshine moor moot mop mope moralise moralize morph mortar mortgage
mortice mortify mortise mosey mosh mothball mother mothproof motion motivate
motley motor motorbike motorboat motorcycle motorise motorize mottle mould
moulder moult mound mount mountaineer mourn mouse mousse mouth move mow muck
muckrake mud mud-wrestle muddle muddy mudwrestle muff muffle mug mulch mulct
mull multiply mumble mummify munch munition murder murk murmur muscle muse mush
mushroom muss mussitate muster mutate mute mutilate mutiny mutter muzzle mystify
mythicise mythicize mythologise mythologize nab nag nail name namedrop nap
narcotise narcotize nark narrate narrow nasale nasalise nasalize nationalise
nationalize natter naturalise naturalize nauseate navigate nazify near neaten
necessitate neck necrose need needle negate negative neglect negociate negotiate
neigh neighbor neighbour nerve nest nestle net netmail nett nettle network
neuter neutralise neutralize nibble nick nickel nickel-and-dime nicker nickname
nictate nictitate niggle nigrify nip nitpick nitrate nitrify nitrogenise
nitrogenize nix nobble nock nod noise nol.pros. nominate nonplus noose normalise
normalize nose nosedive nosh notarise notarize notate notch note notice notify
nourish novate novelise novelize nucleate nudge nuke nullify numb number
numerate nurse nurture nut nutate nutrify nuzzle o.d. o.k. obey obfuscate object
objectify objurgate obligate oblige obliterate obnubilate obscure observe obsess
obsolesce obstinate obstipate obstruct obtain obtrude obtund obturate obviate
occasion occidentalise occidentalize occlude occult occupy occur odorize
odourise off offend offer officer officialise officialize officiate offload
offsaddle offset ogle oil oink okay omen omit one-step ooh ooze opacify opalesce
opalise opalize open operate opine oppose oppress oppugn opsonize opt optimise
optimize orate orb orbit orchestrate ordain order ordinate organise organize
orient orientalise orientalize orientate originate ornament orphan oscillate
osculate ossify ostentate ostracise ostracize oust out out-herod outbalance
outbid outbrave outcall outclass outcrop outcry outdistance outdo outdraw
outface outfight outfit outflank outfox outgeneral outgo outgrow outguess
outlast outlaw outline outlive outmaneuver outmanoeuvre outmarch outmatch
outmode outnumber outpace outperform outplay outpoint output outrage outrange
outrank outride outrival outroar outrun outsail outscore outsell outshine
outshout outsmart outsource outspan outstare outstay outstrip outvie outvote
outwear outweigh outwit ovenbake over-correct over-refine overachieve overact
overarch overawe overbalance overbear overbid overboil overburden overcapitalise
overcapitalize overcast overcharge overclothe overcloud overcome overcompensate
overcook overcrop overcrowd overcultivate overdo overdose overdramatise
overdramatize overdraw overdress overdrive overeat overemphasise overemphasize
overestimate overexert overexploit overexpose overextend overfatigue overfeed
overfill overflow overfly overgeneralise overgeneralize overgorge overgrow
overhang overhaul overhear overheat overindulge overjoy overlap overlay overleap
overlie overload overlook overmaster overpay overplay overpopulate overpower
overpraise overprice overprint overproduce overprotect overrate overreach
overreact overrefine override overrule overrun oversee oversew overshadow
overshoot oversimplify oversleep overspecialise overspecialize overspend
overspread overstate overstay overstep overstock overstrain overstress
overstretch overstuff oversupply overtake overtax overthrow overtire overtop
overtrump overturn overuse overvalue overweary overwhelm overwinter overwork
overwrite ovulate owe own oxidate oxidise oxidize oxygenate oxygenise oxygenize
oyster ozonise ozonize pace pacify pack package pad paddle padlock paganise
paganize page paginate pain paint pair pal palatalise palatalize palaver pale
palisade pall palliate palm palpate palpebrate palpitate palsy palter pamper pan
pan-broil pan-fry pander panel panhandle panic pant pant-hoot pantomime paper
par parachute parade paragraph parallel parallel-park parallelize paralyse
paralyze paraphrase parboil parcel parch pardon pare parent parget park parlay
parley parody parole parrot parry parse part partake parti-color participate
particularise particularize partition partner party pass paste pasteurise
pasteurize pasture pat patch patent patinate patinise patinize patrol patronage
patronise patronize patter pattern pattern-bomb pauperise pauperize pause pave
paw pawn pay peach peak peal pearl peck peculate pedal peddle pedicure pee pee-
pee peek peel peep peer peeve peg pelt pen penalise penalize pencil penetrate
penny-pinch pension people pepper peptise peptize perambulate perceive perch
percolate percuss peregrinate perennate perfect perforate perform perfume
perfuse peril perish perjure perk perm permeate permit permute perorate peroxide
perpetrate perpetuate perplex persecute perseverate persevere persist
personalise personalize personate personify perspire persuade pertain perturb
peruse pervade pervaporate pervert pester pestle pet petition petrify pettifog
phase philander philosophise philosophize phlebotomise phlebotomize phonate
phone phosphoresce photocopy photograph photosensitise photosensitize photostat
phrase pick picket pickle picnic picture piddle piece piece-dye pierce piffle
pig pigeonhole piggyback pigment pile pilfer pillage pillory pillow pilot pimp
pin pinch pine ping pinion pink pinkify pinnacle pinpoint pioneer pip pipe pipe-
clay pique pirate pirouette piss pistol-whip pit pitch pitchfork pith pitter-
patter pity pivot placard placate place place-kick plagiarise plagiarize plague
plain plait plan plane plank plant plash plaster plasticise plasticize
plastinate plat plate platinize platitudinize play playact plea-bargain pleach
plead please pleat pledge plicate plight plod plonk plop plot plough plow pluck
plug plumb plume plummet plump plunder plunge plunk pluralise pluralize ply
poach pock pocket pockmark pod podcast poetise poetize point poise poison poke
polarise polarize pole poleax poleaxe polemicise polemicize polemise polemize
police polish politicise politicize politick polka poll pollard pollenate
pollinate pollute polychrome polychromise polychromize polymerise polymerize
pomade pommel pompadour ponder poniard pontificate pooch pooh-pooh pool pop
popularise popularize populate porcelainize pore port portend porter portion
portray pose posit position possess post postdate postmark postpone postpose
postulate posture pot potentiate pother potter pouch poultice pounce pound pour
pout powder powderise powderize power power-dive powerwash powwow practice
practise praise prance prang prank prate prattle prawn pray preach preachify
preamble prearrange preassemble precede precess precipitate precis preclude
preconceive precondition precook predate predecease predestinate predestine
predetermine predicate predict predigest predispose predominate preempt preen
preexist prefabricate preface prefer prefigure prefix preform preheat prehend
preisolate prejudge prejudice prelude premeditate premier premiere premise
premiss preoccupy preordain prepare prepay preponderate prepose prepossess
prerecord presage prescribe present preserve preside press pressure pressure-
cook pressure-wash pressurise pressurize presume presuppose pretend pretermit
prettify prevail prevaricate prevent preview previse prey price prick prickle
pride prim prime primp prink print prioritise prioritize prise privatise
privatize privilege prize prizefight probate probe proceed process proclaim
procrastinate procreate proctor procure prod produce profane profess
professionalise professionalize proffer profile profit profiteer prognosticate
program programme progress prohibit project prolapse proliferate prologise
prologize prologuize prolong promenade promise promote prompt promulgate pronate
pronk pronounce proof proofread prop propagandise propagandize propagate propel
prophesy propitiate proportion propose proposition propound prorate prorogue
proscribe prosecute proselytise proselytize prospect prosper prostitute
prostrate protect protest protract protrude protuberate prove provide provision
provoke prowl prune pry psalm psychoanalyse psychoanalyze ptyalise ptyalize pub-
crawl publicise publicize publish pucker puddle puff puke pule pull pullulate
pulp pulsate pulse pulverise pulverize pumice pummel pump pun punch punctuate
puncture punish punt pup pupate purchase puree purge purify purl purloin purple
purport purpose purpurate purr purse pursue purvey push pussyfoot put putrefy
putt putter putty puzzle pyramid quack quadruple quadruplicate quaff quail quake
qualify quantify quantise quantize quarantine quarrel quarry quarter quarterback
quash quaver queen queer quell quench query quest question quetch queue quibble
quick-freeze quicken quickstep quiesce quiet quieten quilt quintuple quip quirk
quit quiver quiz quote rabbet rabbit race rack racket racketeer raddle radiate
radicalize radio radiolocate raffle raft rafter rag rage raid rail railroad
raiment rain raise rake rally ram ramble ramify ramp rampage ranch randomise
randomize range rank rankle ransack ransom rant rap rape rappel rarefy rarify
rase rasp rasterize rat ratchet rate ratify ratiocinate ration rationalise
rationalize rattle ravage rave ravel raven ravish ray raze razor razz re-address
re-afforest re-argue re-arm re-assume re-create re-emerge re-emphasise re-
emphasize re-enter re-equip re-examine re-explain re-explore re-incorporate re-
introduce re-start reabsorb reach reacquaint react reactivate read readapt
readjust readmit ready reaffirm realign realine realise realize reallocate
reallot ream reanimate reap reappear reapportion reappraise rear rear-end rearm
rearrange reason reassail reassemble reassert reassess reassign reassure
reattribute reave reawaken rebate rebel rebind reboot rebound rebroadcast rebuff
rebuild rebuke rebury rebut recalcitrate recalculate recall recant recap
recapitulate recapture recast recede receipt receive recess recharge recidivate
reciprocate recite reckon reclaim reclassify recline recode recognise recognize
recoil recollect recombine recommence recommend recommit recompense reconcile
recondition reconfirm reconnoiter reconnoitre reconquer reconsecrate reconsider
reconstitute reconstruct reconvene reconvert reconvict recopy record recount
recoup recover recreate recriminate recrudesce recruit rectify recumb recuperate
recur recurve recuse recycle red-eye red-ink redact redden rede redecorate
rededicate redeem redefine redeploy redeposit redesign redetermine redevelop
redirect rediscover redispose redistribute redline redo redouble redound redress
reduce reduplicate reecho reef reek reel reelect reenact reenforce reestablish
reevaluate reeve reexamine reface refashion refer referee reference refill
refinance refine refinish refit reflate reflect reflectorise reflectorize
refloat refocus reforest reforge reform reformulate refract refracture refrain
refresh refreshen refrigerate refuel refund refurbish refurnish refuse refute
regain regale regard regenerate regiment register regorge regress regret regroup
regrow regularise regularize regulate regurgitate rehabilitate reharmonise
reharmonize rehash rehear rehearse reheat reheel rehouse reify reign reignite
reimburse reimpose rein reincarnate reinforce reinstall reinstate reinsure
reintegrate reinterpret reintroduce reinvent reinvigorate reissue reiterate
reject rejig rejoice rejoin rejuvenate rekindle relace relapse relate relativise
relativize relax relay relearn release relegate relent relieve reline relinquish
relish relive reload relocate rely relyric remain remainder remake remand remark
remarry remediate remedy remember remilitarise remilitarize remind reminisce
remit remodel remold remonstrate remould remount remove remunerate rename rend
render render-set rendezvous renegade renege renegociate renegotiate renew
renormalise renormalize renounce renovate rent reopen reorder reorganise
reorganize reorient reorientate repaint repair repatriate repay repeal repeat
repel repent repercuss rephrase repine replace replant replay replenish replete
replicate reply repoint report repose reposit reposition repossess repot
reprehend represent repress reprieve reprimand reprint reprise reprize reproach
reprobate reprocess reproduce reproof reprove republish repudiate repugn repulse
repurchase repute request require requisition requite reread rerun rescale
reschedule rescind rescue reseal research reseat resect reseed resell resemble
resent reserve reset resettle resew reshape reship reshoot reshuffle reside
resift resign resile resinate resist resize resmudge resole resolve resonate
resorb resort resound respect respire respite resplend respond rest restart
restate restitute restock restore restrain restrengthen restrict restructure
resublime resubmit result resume resurface resurge resurrect resuscitate
resuspend ret retail retain retake retaliate retard retch retell rethink retick
reticulate retie retire retool retort retouch retrace retract retrain
retranslate retransmit retread retreat retrench retrieve retrofit retroflex
retrograde retrogress retrospect retrovert retry return reunify reunite reuse
rev revalue revamp reveal revel revenge reverberate revere reverence reverse
revert revet review revile revise revisit revitalise revitalize revive revivify
revoke revolt revolutionise revolutionize revolve reward rewire reword rework
rewrite rhapsodise rhapsodize rhumba rhyme rib rice rick ricochet rid riddle
ride ridge ridicule riff riffle rifle rig right rigidify rile rim rime ring
rinse riot rip ripen riposte ripple rise risk ritualise ritualize rival rive
rivet roach roam roar roast rob robe rock rocket roil roister roleplay roll
rollerblade rollick romance romanise romanize romanticise romanticize romp roneo
roof rook room roost root rootle rope rosin rot rotate rouge rough rough-dry
rough-hew rough-house rough-sand roughcast roughen roughhouse round rouse rout
route rove row rub rubber rubberise rubberize rubberneck rubberstamp rubbish
rubify rubric rubricate ruck ruckle ruddle rue ruff ruffle ruggedise ruggedize
ruin rule rumba rumble ruminate rummage rumor rumour rumple rumpus run rupture
rush rust rusticate rustle rut saber sabotage sabre saccharify sack sacrifice
sadden saddle safeguard sag sail sailplane saint salaam salinate salivate sallow
salt saltate salute salvage salve samba sample sanctify sanction sand sandbag
sandblast sandpaper sandwich sanitate sanitise sanitize sap saponify sashay sass
sate satellite satiate satirise satirize satisfice satisfise satisfy saturate
sauce saunter saute savage save savor savour savvy saw say scab scaffold scald
scale scallop scalp scam scamp scamper scan scandalise scandalize scant scar
scare scarf scarify scarper scat scatter scavenge scend scent schedule
schematise schematize scheme schlep schmoose schmooze schnorr school schuss
scintillate scissor sclaff scoff scold scollop scoop scoot scorch score scorn
scotch scour scourge scout scowl scrabble scrag scram scramble scranch scrap
scrape scratch scraunch scrawl screak scream screech screen screw scribble
scribe scrimmage scrimp scrimshank script scroll scrounge scrub scrunch scruple
scrutinise scrutinize scry scud scuff scuffle scull sculpt sculpture scum
scupper scurry scuttle scythe seal seam seaplane sear search season seat secede
secern secernate seclude second second-guess secrete section sectionalise
sectionalize secularise secularize secure sedate sediment seduce see seed seek
seel seem seep seesaw seethe segment segregate segue seine seize select self-
destroy self-destruct sell sellotape semaphore semi-automatise semi-automatize
send senesce sense sensibilise sensibilize sensify sensitise sensitize
sensualise sensualize sentence sentimentalise sentimentalize sentimentise
sentimentize separate sequence sequester sequestrate serenade serialise
serialize sermonise sermonize serrate serve service set settle sever severalise
severalize sew sex sexualise sexualize shack shackle shade shadow shadowbox
shaft shag shake shallow sham shamanise shamanize shamble shame shampoo shanghai
shank shape share shark sharpen sharpshoot shatter shave she-bop shear sheathe
shed sheer sheet shell shellac shellack shelter shelve shepherd shew shield
shift shill shillyshally shimmer shimmy shin shine shingle shinny ship shipwreck
shirk shirr shirt shit shiver shlep shmoose shmooze shnorr shoal shock shoe
shoehorn shoetree shoo shoot shop shoplift shore short short-change short-
circuit shorten shortlist shoulder shout shove shovel show shower shred shriek
shrill shrimp shrine shrink shrinkwrap shrive shrivel shroud shrug shuck shudder
shuffle shun shunt shush shut shutter shuttle shuttlecock shy sibilate sic sick
sicken side side-slip sideline sidestep sideswipe sidetrack sidle sieve sift
sigh sight sight-read sight-sing sightread sightsee sightsing sign signal
signalise signalize signify signpost silence silhouette silkscreen silt silver
silver-plate silverplate simmer simonise simonize simper simplify simulate sin
sing singe single single-foot singsong singularise singularize sink sinter sip
siphon sire siss sit site situate size sizz sizzle skank skate skateboard
skedaddle sketch skew skewer ski skid skim skimcoat skimp skin skin-dive skinny-
dip skip skip-bomb skipper skirl skirmish skirt skitter skittle skive skreak
skreigh skulk skunk sky skydive skyjack skylark skyrocket slabber slack slacken
slag slake slalom slam slam-dunk slander slang slant slap slash slat slate
slather slaughter slave slaver slay sled sledge sledgehammer sleek sleep
sleepwalk sleet sleigh slenderise slenderize sleuth slew slice slick slide
slight slim slime sling slink slip slit slither sliver slobber slog sloganeer
slop slope slosh slot slouch slough slow slue slug sluice slum slumber slump
slur slurp slush smack smart smash smatter smear smell smelt smile smirch smirk
smite smock smoke smolder smooch smooth smoothen smother smoulder smudge smuggle
smut smutch snack snaffle snafu snag snail snake snap snare snarf snarl snatch
sneak sneer sneeze snick snicker sniff sniffle snigger snip snipe snitch snivel
snog snooker snoop snooze snore snorkel snort snow snow-blind snowball snowboard
snowmobile snowshoe snub snuff snuffle snuggle soak soap soar sob sober
socialise socialize sock sod sodomise sodomize soft-pedal soft-soap soft-solder
soften soil sojourn solace solarise solarize solder soldier sole solemnise
solemnize solicit solidify soliloquise soliloquize solmizate solo solvate solve
somersault somnambulate sonnet soot soothe sop sophisticate sorb sorcerise
sorcerize sorrow sort sough sound soundproof soup sour source souse sovietise
sovietize sow space spacewalk spade spam span spang spangle spank spar spare
sparge spark sparkle spat spatchcock spatter spawn spay speak spear spearhead
specialise specialize speciate specify speck speckle spectate speculate speech-
read speechify speed spell spellbind spelunk spend spew sphacelate spice spiel
spike spill spin spin-dry spiral spirit spiritise spiritize spiritualise
spiritualize spirt spit spite splash splat splatter splay splice splint splinter
split splosh splotch splurge splutter spoil spondaise spondaize sponge sponsor
spoof spook spool spoon spoonfeed sport sportscast sporulate spot spot-check
spot-weld spotlight spotweld spout sprain sprawl spray spray-dry spread spread-
eagle spreadeagle spree spring spring-clean sprinkle sprint spritz sprout spruce
spud spue spume spur spurn spurt sputter spy squabble squall squander square
squash squat squawk squeak squeal squeegee squeeze squelch squinch squint squire
squirm squirt squish stab stabilise stabilize stable stack staff stag stage
stagger stagnate stain stake stale stalemate stalinise stalinize stalk stall
stammer stamp stampede stanch stand standardise standardize staple star
starboard starch stare stargaze start startle starve stash state station staunch
stave stay steady steal steam steam-heat steamer steamroll steamroller steel
steep steepen steer stem stencil stenograph step stereotype sterilise sterilize
stet stew stick stickle stiffen stifle stigmatise stigmatize still still-fish
still-hunt stimulate sting stink stint stipple stipulate stir stitch stock
stock-take stockade stockpile stoke stomach stomp stone stone-wash stonewall
stonewash stooge stool stoop stop stopper stopple store storm stow straddle
strafe straggle straighten strain straiten strand strangle strangulate strap
stratify straw stray streak stream streamline street-walk streetwalk strengthen
stress stretch strew striate strickle stride stridulate strike string strip
strip-search stripe strive stroke stroll strong-arm strop structure struggle
strum strut stub stucco stud study stuff stultify stumble stump stun stunt
stupefy stutter style stylise stylize stymie stymy sub subcontract subdivide
subdue subedit subject subjoin subjugate sublease sublet sublimate sublime
subluxate submarine submerge submerse submit subordinate suborn subpoena
subrogate subscribe subserve subside subsidise subsidize subsist substantiate
substitute subsume subtend subtilise subtilize subtitle subtract suburbanise
suburbanize subvent subvention subvert subvocalise subvocalize succeed succor
succour succumb succuss suck suckle suction sudate suds sue suffer suffice
suffix suffocate suffuse sugar sugarcoat suggest suit sulfate sulfur sulfurette
sulk sully sulphur sulphurette sum summarise summarize summate summer summerise
summerize summit summon summons sun sunbathe sunburn sunder suntan sup
superannuate supercede supercharge superfetate superimpose superinfect
superintend superordinate superpose superscribe supersede supervene supervise
supinate supplant supple supplement supplicate supply support suppose suppress
suppurate surcharge surf surface surface-mine surfboard surfeit surge surmise
surmount surpass surprise surrender surround surtax surveil survey survive
suspect suspend suspire sustain susurrate suture swab swaddle swag swage swagger
swallow swamp swan swank swap swarm swash swat swathe swatter sway swear sweat
sweep sweet-talk sweeten swell swelter swerve swig swill swim swindle swing
swinge swipe swirl swish switch switch-hit swivel swob swoon swoop swoosh swop
swosh swot syllabicate syllabify syllabise syllabize syllogise syllogize
symbolise symbolize symmetrise symmetrize sympathise sympathize symphonise
symphonize sync synchronise synchronize syncopate syncretise syncretize
syndicate synthesise synthesize syphon syringe systematise systematize systemise
systemize table taboo tabularise tabularize tabulate tack tackle tag tail
tailgate tailor tailor-make taint take talc talk tally tame tamp tamper tampon
tan tangle tango tank tantalise tantalize tap tapdance tape taper tar tar-and-
feather target tariff tarmac tarnish tarry task taste tat tattle tattoo taunt
tauten tax taxi teach team tear teargas tease tee teem teeter teeter-totter
teetertotter teethe teetotal telecast telecommunicate telefax telegraph
telepathise telepathize telephone teleport telescope televise telex tell temper
temporise temporize tempt tenant tend tender tenderise tenderize tense tent
tenure tergiversate term terminate terrace terrasse terrify territorialise
territorialize terrorise terrorize tessellate test testify tether thank thatch
thaw theme theologise theologize theorise theorize thermostat thicken thieve
thin think thirst thoriate thrash thread threaten thresh thrill thrive throb
thrombose throne throng throttle throw thrum thrust thud thumb thumbtack thump
thunder thurify thwack thwart tick ticket tickle ticktack ticktock tide tidy tie
tie-dye tighten tile till tiller tilt time tin tinct tincture ting tinge tingle
tink tinker tinkle tinsel tint tintinnabulate tip tipple tippytoe tiptoe tire
tissue tithe titillate titivate title titrate titter tittivate tittle-tattle
tittup toady toast toboggan toddle toe toenail tog toggle toil toilet-train
tolerate toll tomahawk tone tongue tongue-tie tonsure tool toot tootle top
topdress tope topicalize topple torch torment torpedo torture toss tot total
totalise totalize tote totter touch touch-type toughen tour tourney tousle tout
tow towel tower toy trace track trade trademark traduce traffic trail train
traipse tram trammel tramp trample trance tranquilize tranquillise tranquillize
transact transaminate transcend transcribe transduce transect transfer
transfigure transfix transform transfuse transgress transistorise transistorize
transit transition transitivise transitivize translate transliterate translocate
transmigrate transmit transmogrify transmute transpirate transpire transplant
transport transpose transship transubstantiate transude trap trash traumatise
traumatize travail travel traverse travesty trawl tread treadle treasure treat
treble tree trek trellis tremble tremor trench trend trepan trephine trespass
triangulate tribulate trice trick trickle trifle trifurcate trigger trill trim
trip triple triple-space triple-tongue triplicate trisect triumph trivialise
trivialize troat troll troop trot trouble trouble-shoot troubleshoot trounce
trowel truck truckle trudge true trump trumpet truncate trundle truss trust try
tsk tube tuck tucker tug tumble tumefy tumesce tune tunnel turf turn turtle tusk
tussle tut tut-tut tutor twaddle twang tweak tweedle tweet tweeze twiddle twig
twill twin twine twinge twinkle twirl twirp twist twit twitch twitter two-step
two-time type typecast typeset typewrite typify tyrannise tyrannize uglify
ulcerate ultracentrifuge ululate umpire unarm unbalance unbar unbelt unbend
unbind unblock unbolt unbosom unbox unbrace unbraid unbridle unbuckle unburden
unbutton uncase unchain unchurch unclasp unclip uncloak unclog unclothe
unclutter uncoil uncompress uncork uncouple uncover uncrate uncross uncurl
undeceive underachieve underact underbid undercharge undercoat undercut
underdevelop underdress underestimate underexpose undergird undergo undergrow
underlay underlie underline undermine undernourish underpay underperform
underpin underplay underprice underproduce underquote underrate underscore
undersell undershoot undersign underspend understand understate understock
understudy undertake undervalue underwrite undo undock undrape undress undulate
unearth unfasten unfit unfold unfreeze unfrock unfurl unhallow unhand unharness
unhinge unhitch unhook unhorse unicycle uniform uniformise uniformize unify
unionise unionize unite unitise unitize universalise universalize unknot unlace
unlade unlash unlax unlearn unleash unlive unload unlock unloose unloosen unmake
unman unmask unmuzzle unnerve unpack unpick unpin unplug unravel unreel unroll
unsaddle unsanctify unsay unscramble unscrew unseal unseat unsettle unsex
unsheathe unsnarl unsolder unspell unstaple unstrain unstrap unstring unstuff
unsubstantialise unsubstantialize untangle unteach unthaw untie untune untwine
untwist unveil unweave unwind unwire unwrap unyoke unzip up upbraid upchuck
update upend upgrade upheave uphold upholster uplift upload upraise uprise
uproot upset upstage urbanise urbanize urge urinate urticate use usher usurp
utilise utilize utter vacate vacation vaccinate vacillate vacuum vacuum-clean
vagabond valet validate valuate value vamoose vamp vandalise vandalize vanish
vanquish vaporise vaporize variegate varnish vary vascularise vascularize
vasectomise vasectomize vaticinate vault vaunt veer vegetate veil vein velcro
vellicate vend veneer venerate venesect vent ventilate venture verbalise
verbalize verbify verdigris verge verify vermiculate vermilion verse versify
vesicate vesiculate vest vesture vet veto vex vibrate victimise victimize
victual videotape vie view vilify vilipend vindicate vinify violate virilise
virilize visa visit visualise visualize vitalise vitalize vitaminise vitaminize
vitiate vitrify vitriol vituperate vivify vivisect vocalise vocalize vociferate
voice void volatilise volatilize volley volunteer vomit voodoo vote vouch
vouchsafe vow vowelise vowelize voyage vroom vulcanise vulcanize vulgarise
vulgarize wad waddle wade waffle waft wag wage wager waggle wail wait waitress
waive wake waken walk wall wallop wallow wallpaper waltz wamble wan wander wane
wangle wank want wanton war warble ward ware warehouse warm warn warp warrant
wash wassail waste watch water water-wash watercolor watercolour waterproof
wattle waul wave waver wawl wax waylay weaken wean weaponize wear weary weather
weatherproof weatherstrip weave web wed wedel wedge wee wee-wee weed weekend
weep weigh weight weight-lift weightlift welch welcome weld well welsh welt
welter wench wend westernise westernize wet wet-nurse whack whale wham whang
whap wharf wheedle wheel wheelbarrow wheeze whelk whelm whelp whet whicker whiff
whimper whine whinny whip whipsaw whir whirl whirligig whirlpool whirr whish
whisk whisker whisper whistle whistlestop white white-out whiten whiteout
whitewash whittle whiz whizz wholesale whomp whoop whoosh whop whore widen widow
wield wiggle wigwag will wilt win wince winch wind window-dress window-shop
windsurf wine wing wink winkle winnow winter winterise winterize wipe wire
wiretap wisecrack wish witch withdraw wither withhold withstand witness wive
wobble wolf wolf-whistle womanise womanize wonder woo woolgather woosh word work
worm worry worsen worship worst wound wow wrack wrangle wrap wrawl wreak wreathe
wreck wrench wrest wrestle wrick wriggle wring wrinkle write writhe wrong x-ray
xerox yacht yack yak yammer yank yap yarn yarn-dye yaup yaw yawl yawn yawp yearn
yell yellow yelp yen yield yip yodel yoke yowl zap zero zest zigzag zinc zip
zipper zone zoom
""".split())



# coding: utf-8
from __future__ import unicode_literals

import pytest


@pytest.mark.parametrize('string,lemma', [('Abgehängten', 'Abgehängte'),
                                          ('engagierte', 'engagieren'),
                                          ('schließt', 'schließen'),
                                          ('vorgebenden', 'vorgebend')])
def test_lemmatizer_lookup_assigns(de_tokenizer, string, lemma):
    tokens = de_tokenizer(string)
    assert tokens[0].lemma_ == lemma



# coding: utf-8
from __future__ import unicode_literals

import numpy
import pytest


@pytest.fixture
def example(EN):
    """
    This is to make sure the model works as expected. The tests make sure that
    values are properly set. Tests are not meant to evaluate the content of the
    output, only make sure the output is formally okay.
    """
    assert EN.entity != None
    return EN('There was a stranger standing at the big street talking to herself.')


@pytest.mark.models('en')
def test_en_models_tokenization(example):
    # tokenization should split the document into tokens
    assert len(example) > 1


@pytest.mark.models('en')
def test_en_models_tagging(example):
    # if tagging was done properly, pos tags shouldn't be empty
    assert example.is_tagged
    assert all(t.pos != 0 for t in example)
    assert all(t.tag != 0 for t in example)


@pytest.mark.models('en')
def test_en_models_parsing(example):
    # if parsing was done properly
    # - dependency labels shouldn't be empty
    # - the head of some tokens should not be root
    assert example.is_parsed
    assert all(t.dep != 0 for t in example)
    assert any(t.dep != i for i,t in enumerate(example))


@pytest.mark.models('en')
def test_en_models_ner(example):
    # if ner was done properly, ent_iob shouldn't be empty
    assert all([t.ent_iob != 0 for t in example])


@pytest.mark.models('en')
def test_en_models_vectors(example):
    # if vectors are available, they should differ on different words
    # this isn't a perfect test since this could in principle fail
    # in a sane model as well,
    # but that's very unlikely and a good indicator if something is wrong
    if example.vocab.vectors_length:
        vector0 = example[0].vector
        vector1 = example[1].vector
        vector2 = example[2].vector
        assert not numpy.array_equal(vector0,vector1)
        assert not numpy.array_equal(vector0,vector2)
        assert not numpy.array_equal(vector1,vector2)


@pytest.mark.xfail
@pytest.mark.models('en')
def test_en_models_probs(example):
    # if frequencies/probabilities are okay, they should differ for
    # different words
    # this isn't a perfect test since this could in principle fail
    # in a sane model as well,
    # but that's very unlikely and a good indicator if something is wrong
    prob0 = example[0].prob
    prob1 = example[1].prob
    prob2 = example[2].prob
    assert not prob0 == prob1
    assert not prob0 == prob2
    assert not prob1 == prob2


@pytest.mark.models('en')
def test_no_vectors_similarity(EN):
    doc1 = EN(u'hallo')
    doc2 = EN(u'hi')
    assert doc1.similarity(doc2) > 0




# coding: utf-8
from __future__ import unicode_literals

from ..util import ensure_path
from .. import util
from ..displacy import parse_deps, parse_ents
from ..tokens import Span
from .util import get_doc
from .._ml import PrecomputableAffine

from pathlib import Path
import pytest
from thinc.neural._classes.maxout import Maxout
from thinc.neural._classes.softmax import Softmax
from thinc.api import chain


@pytest.mark.parametrize('text', ['hello/world', 'hello world'])
def test_util_ensure_path_succeeds(text):
    path = util.ensure_path(text)
    assert isinstance(path, Path)


@pytest.mark.parametrize('package', ['numpy'])
def test_util_is_package(package):
    """Test that an installed package via pip is recognised by util.is_package."""
    assert util.is_package(package)


@pytest.mark.parametrize('package', ['thinc'])
def test_util_get_package_path(package):
    """Test that a Path object is returned for a package name."""
    path = util.get_package_path(package)
    assert isinstance(path, Path)


def test_displacy_parse_ents(en_vocab):
    """Test that named entities on a Doc are converted into displaCy's format."""
    doc = get_doc(en_vocab, words=["But", "Google", "is", "starting", "from", "behind"])
    doc.ents = [Span(doc, 1, 2, label=doc.vocab.strings[u'ORG'])]
    ents = parse_ents(doc)
    assert isinstance(ents, dict)
    assert ents['text'] == 'But Google is starting from behind '
    assert ents['ents'] == [{'start': 4, 'end': 10, 'label': 'ORG'}]


def test_displacy_parse_deps(en_vocab):
    """Test that deps and tags on a Doc are converted into displaCy's format."""
    words = ["This", "is", "a", "sentence"]
    heads = [1, 0, 1, -2]
    pos = ['DET', 'VERB', 'DET', 'NOUN']
    tags = ['DT', 'VBZ', 'DT', 'NN']
    deps = ['nsubj', 'ROOT', 'det', 'attr']
    doc = get_doc(en_vocab, words=words, heads=heads, pos=pos, tags=tags,
                  deps=deps)
    deps = parse_deps(doc)
    assert isinstance(deps, dict)
    assert deps['words'] == [{'text': 'This', 'tag': 'DET'},
                            {'text': 'is', 'tag': 'VERB'},
                            {'text': 'a', 'tag': 'DET'},
                            {'text': 'sentence', 'tag': 'NOUN'}]
    assert deps['arcs'] == [{'start': 0, 'end': 1, 'label': 'nsubj', 'dir': 'left'},
                            {'start': 2, 'end': 3, 'label': 'det', 'dir': 'left'},
                            {'start': 1, 'end': 3, 'label': 'attr', 'dir': 'right'}]


def test_PrecomputableAffine(nO=4, nI=5, nF=3, nP=2):
    model = PrecomputableAffine(nO=nO, nI=nI, nF=nF, nP=nP)
    assert model.W.shape == (nF, nO, nP, nI)
    tensor = model.ops.allocate((10, nI))
    Y, get_dX = model.begin_update(tensor)
    assert Y.shape == (tensor.shape[0]+1, nF, nO, nP)
    assert model.d_pad.shape == (1, nF, nO, nP)
    dY = model.ops.allocate((15, nO, nP))
    ids = model.ops.allocate((15, nF))
    ids[1,2] = -1
    dY[1] = 1
    assert model.d_pad[0, 2, 0, 0] == 0.
    model._backprop_padding(dY, ids)
    assert model.d_pad[0, 2, 0, 0] == 1.
    model.d_pad.fill(0.)
    ids.fill(0.)
    dY.fill(0.)
    ids[1,2] = -1
    ids[1,1] = -1
    ids[1,0] = -1
    dY[1] = 1
    assert model.d_pad[0, 2, 0, 0] == 0.
    model._backprop_padding(dY, ids)
    assert model.d_pad[0, 2, 0, 0] == 3.
